# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path as url
from codenerix_invoicing.views_purchases import ProviderList, BudgetList, LineBudgetList, \
    OrderList, LineOrderList, AlbaranList, LineAlbaranList, \
    TicketList, LineTicketList, TicketRectificationList, \
    LineTicketRectificationList, InvoiceList, LineInvoiceList, \
    InvoiceRectificationList, LineInvoiceRectificationList, \
    ProviderCreate, \
    BudgetCreate, OrderCreate, AlbaranCreate, TicketCreate, \
    TicketRectificationCreate, InvoiceCreate, \
    InvoiceRectificationCreate, \
    ProviderCreateModal, \
    BudgetCreateModal, LineBudgetCreateModal, OrderCreateModal, \
    LineOrderCreateModal, AlbaranCreateModal, LineAlbaranCreateModal, \
    TicketCreateModal, LineTicketCreateModal, TicketRectificationCreateModal, \
    LineTicketRectificationCreateModal, InvoiceCreateModal, LineInvoiceCreateModal, InvoiceRectificationCreateModal, LineInvoiceRectificationCreateModal, \
    ProviderUpdate, ProviderDetails, \
    BudgetUpdate, OrderUpdate, AlbaranUpdate, \
    TicketUpdate, TicketRectificationUpdate, \
    InvoiceUpdate, \
    InvoiceRectificationUpdate, ProviderUpdateModal, \
    BudgetUpdateModal, LineBudgetUpdateModal, OrderUpdateModal, \
    LineOrderUpdateModal, AlbaranUpdateModal, LineAlbaranUpdateModal, \
    TicketUpdateModal, LineTicketUpdateModal, TicketRectificationUpdateModal,\
    LineTicketRectificationUpdateModal, InvoiceUpdateModal, LineInvoiceUpdateModal, \
    InvoiceRectificationUpdateModal, LineInvoiceRectificationUpdateModal, \
    ProviderDelete, \
    BudgetDelete, LineBudgetDelete, OrderDelete, LineOrderDelete, \
    AlbaranDelete, LineAlbaranDelete, TicketDelete, LineTicketDelete, \
    TicketRectificationDelete, LineTicketRectificationDelete, InvoiceDelete, \
    LineInvoiceDelete, InvoiceRectificationDelete, LineInvoiceRectificationDelete, \
    BudgetDocumentList, OrderDocumentList, AlbaranDocumentList, \
    TicketDocumentList, TicketRectificationDocumentList, InvoiceDocumentList, \
    InvoiceRectificationDocumentList, BudgetDocumentCreateModal, OrderDocumentCreateModal, AlbaranDocumentCreateModal, \
    TicketDocumentCreateModal, TicketRectificationDocumentCreateModal, InvoiceDocumentCreateModal, InvoiceRectificationDocumentCreateModal, \
    BudgetDocumentUpdateModal, OrderDocumentUpdateModal, AlbaranDocumentUpdateModal, \
    TicketDocumentUpdateModal, TicketRectificationDocumentUpdateModal, InvoiceDocumentUpdateModal, InvoiceRectificationDocumentUpdateModal, \
    BudgetDocumentDelete, OrderDocumentDelete, AlbaranDocumentDelete, \
    TicketDocumentDelete, TicketRectificationDocumentDelete, InvoiceDocumentDelete, InvoiceRectificationDocumentDelete, \
    BudgetDocumentSubList, OrderDocumentSubList, AlbaranDocumentSubList, \
    TicketDocumentSubList, TicketRectificationDocumentSubList, InvoiceDocumentSubList, InvoiceRectificationDocumentSubList, \
    BudgetDocumentDetails, OrderDocumentDetails, AlbaranDocumentDetails, \
    TicketDocumentDetails, TicketRectificationDocumentDetails, InvoiceDocumentDetails, \
    InvoiceRectificationDocumentDetails, \
    BudgetDocumentDetailsModal, OrderDocumentDetailsModal, AlbaranDocumentDetailsModal, \
    TicketDocumentDetailsModal, TicketRectificationDocumentDetailsModal, InvoiceDocumentDetailsModal, InvoiceRectificationDocumentDetailsModal, \
    BudgetDetails, LineBudgetSubList, LineBudgetDetailsModal, \
    OrderDetails, OrderForeign, LineOrderSubList, LineOrderDetailsModal, \
    AlbaranDetails, LineAlbaranSubList, LineAlbaranDetailsModal, \
    InvoiceDetails, LineInvoiceSubList, LineInvoiceDetailsModal, \
    InvoiceRectificationDetails, LineInvoiceRectificationSubList, LineInvoiceRectificationDetailsModal, \
    TicketDetails, LineTicketSubList, LineTicketDetailsModal, \
    TicketRectificationDetails, LineTicketRectificationSubList, LineTicketRectificationDetailsModal, \
    BudgetDoOrder, OrderCreateFromBudget, \
    LineBudgetSubListForm, OrderDetailsModal, \
    BudgetPrint, OrderPrint, AlbaranPrint, TicketPrint, TicketRectificationPrint, InvoicePrint, InvoiceRectificationPrint

from codenerix_invoicing.views_purchases import ProductFinalOfBudgetPurchase    


urlpatterns = [
    url(r'^providers$', ProviderList.as_view(), name='CDNX_invoicing_providers_list'),
    url(r'^providers/add$', ProviderCreate.as_view(), name='CDNX_invoicing_providers_add'),
    url(r'^providers/addmodal$', ProviderCreateModal.as_view(), name='CDNX_invoicing_providers_addmodal'),
    url(r'^providers/(?P<pk>\w+)$', ProviderDetails.as_view(), name='CDNX_invoicing_providers_details'),
    url(r'^providers/(?P<pk>\w+)/edit$', ProviderUpdate.as_view(), name='CDNX_invoicing_providers_edit'),
    url(r'^providers/(?P<pk>\w+)/editmodal$', ProviderUpdateModal.as_view(), name='CDNX_invoicing_providers_editmodal'),
    url(r'^providers/(?P<pk>\w+)/delete$', ProviderDelete.as_view(), name='CDNX_invoicing_providers_delete'),

    url(r'^budgets$', BudgetList.as_view(), name='CDNX_invoicing_budgetpurchasess_list'),
    url(r'^budgets/add$', BudgetCreate.as_view(), name='CDNX_invoicing_budgetpurchasess_add'),
    url(r'^budgets/addmodal$', BudgetCreateModal.as_view(), name='CDNX_invoicing_budgetpurchasess_addmodal'),
    url(r'^budgets/(?P<pk>\w+)$', BudgetDetails.as_view(), name='CDNX_invoicing_budgetpurchasess_details'),
    url(r'^budgets/(?P<pk>\w+)/edit$', BudgetUpdate.as_view(), name='CDNX_invoicing_budgetpurchasess_edit'),
    url(r'^budgets/(?P<pk>\w+)/editmodal$', BudgetUpdateModal.as_view(), name='CDNX_invoicing_budgetpurchasess_editmodal'),
    url(r'^budgets/(?P<pk>\w+)/delete$', BudgetDelete.as_view(), name='CDNX_invoicing_budgetpurchasess_delete'),
    url(r'^budgets/(?P<pk>\w+)/order$', BudgetDoOrder.as_view(), name='CDNX_invoicing_budgetsaless_order'),
    url(r'^budgets/(?P<cpk>\w+)/doorder/(?P<pk>\w+)$', OrderCreateFromBudget.as_view(), name='CDNX_invoicing_budgetsaless3_order'),
    url(r'^budgets/(?P<pk>\w+)/print$', BudgetPrint.as_view(), name='CDNX_invoicing_budgetpurchasess_print'),

    url(r'^purchasesbudgets/addmodal$', BudgetCreateModal.as_view(), name='CDNX_invoicing_budgetpurchasess_addmodal_l'),

    url(r'^linebudgets$', LineBudgetList.as_view(), name='CDNX_invoicing_linebudgetpurchasess_list'),
    url(r'^linebudgets/(?P<pk>\w+)/delete$', LineBudgetDelete.as_view(), name='CDNX_invoicing_linebudgetpurchasess_delete'),
    url(r'^linebudgets/(?P<pk>\w+)/sublist$', LineBudgetSubList.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist'),
    url(r'^linebudgets/(?P<pk>\w+)/sublistform$', LineBudgetSubListForm.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_form'),
    url(r'^linebudgets/(?P<pk>\w+)/sublist/add$', LineBudgetCreateModal.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_add'),
    url(r'^linebudgets/(?P<pk>\w+)/sublist/addmodal$', LineBudgetCreateModal.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_addmodal'),
    url(r'^linebudgets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineBudgetDetailsModal.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_details'),
    url(r'^linebudgets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineBudgetUpdateModal.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_edit'),
    url(r'^linebudgets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineBudgetUpdateModal.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_editmodal'),
    url(r'^linebudgets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineBudgetDelete.as_view(), name='CDNX_invoicing_linebudgetpurchasess_sublist_delete'),

    url(r'^orders$', OrderList.as_view(), name='CDNX_invoicing_orderpurchasess_list'),
    url(r'^orders/add$', OrderCreate.as_view(), name='CDNX_invoicing_orderpurchasess_add'),
    url(r'^orders/addmodal$', OrderCreateModal.as_view(), name='CDNX_invoicing_orderpurchasess_addmodal'),
    url(r'^orders/(?P<pk>\w+)$', OrderDetails.as_view(), name='CDNX_invoicing_orderpurchasess_details'),
    url(r'^orders#/(?P<pk>\w+)$', OrderDetails.as_view(), name='CDNX_invoicing_orderpurchasess_details_js'),
    url(r'^orders/(?P<pk>\w+)/modal$', OrderDetailsModal.as_view(), name='CDNX_invoicing_orderpurchasess_detailsmodal'),
    url(r'^orders/(?P<pk>\w+)/edit$', OrderUpdate.as_view(), name='CDNX_invoicing_orderpurchasess_edit'),
    url(r'^orders/(?P<pk>\w+)/editmodal$', OrderUpdateModal.as_view(), name='CDNX_invoicing_orderpurchasess_editmodal'),
    url(r'^orders/(?P<pk>\w+)/delete$', OrderDelete.as_view(), name='CDNX_invoicing_orderpurchasess_delete'),
    url(r'^orders/(?P<pk>\w+)/print$', OrderPrint.as_view(), name='CDNX_invoicing_orderpurchasess_print'),
    url(r'^orders/foreign/(?P<search>[\w\W]+|\*)$', OrderForeign.as_view(), name='CDNX_invoicing_orderpurchasess_foreign'),

    url(r'^lineorders$', LineOrderList.as_view(), name='CDNX_invoicing_lineorderpurchasess_list'),
    url(r'^lineorders/(?P<pk>\w+)/delete$', LineOrderDelete.as_view(), name='CDNX_invoicing_lineorderpurchasess_delete'),
    url(r'^lineorders/(?P<pk>\w+)/sublist$', LineOrderSubList.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist'),
    url(r'^lineorders/(?P<pk>\w+)/sublist/add$', LineOrderCreateModal.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_add'),
    url(r'^lineorders/(?P<pk>\w+)/sublist/addmodal$', LineOrderCreateModal.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_addmodal'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineOrderDetailsModal.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_details'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineOrderUpdateModal.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_edit'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineOrderUpdateModal.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_editmodal'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineOrderDelete.as_view(), name='CDNX_invoicing_lineorderpurchasess_sublist_delete'),

    url(r'^albarans$', AlbaranList.as_view(), name='CDNX_invoicing_albaranpurchasess_list'),
    url(r'^albarans/add$', AlbaranCreate.as_view(), name='CDNX_invoicing_albaranpurchasess_add'),
    url(r'^albarans/addmodal$', AlbaranCreateModal.as_view(), name='CDNX_invoicing_albaranpurchasess_addmodal'),
    url(r'^albarans/(?P<pk>\w+)$', AlbaranDetails.as_view(), name='CDNX_invoicing_albaranpurchasess_details'),
    url(r'^albarans/(?P<pk>\w+)/edit$', AlbaranUpdate.as_view(), name='CDNX_invoicing_albaranpurchasess_edit'),
    url(r'^albarans/(?P<pk>\w+)/editmodal$', AlbaranUpdateModal.as_view(), name='CDNX_invoicing_albaranpurchasess_editmodal'),
    url(r'^albarans/(?P<pk>\w+)/delete$', AlbaranDelete.as_view(), name='CDNX_invoicing_albaranpurchasess_delete'),
    url(r'^albarans/(?P<pk>\w+)/print$', AlbaranPrint.as_view(), name='CDNX_invoicing_albaranpurchasess_print'),

    url(r'^linealbarans$', LineAlbaranList.as_view(), name='CDNX_invoicing_linealbaranpurchasess_list'),
    url(r'^linealbarans/(?P<pk>\w+)/delete$', LineAlbaranDelete.as_view(), name='CDNX_invoicing_linealbaranpurchasess_delete'),
    url(r'^linealbarans/(?P<pk>\w+)/sublist$', LineAlbaranSubList.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist'),
    url(r'^linealbarans/(?P<pk>\w+)/sublist/add$', LineAlbaranCreateModal.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_add'),
    url(r'^linealbarans/(?P<pk>\w+)/sublist/addmodal$', LineAlbaranCreateModal.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_addmodal'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineAlbaranDetailsModal.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_details'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineAlbaranUpdateModal.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_edit'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineAlbaranUpdateModal.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_editmodal'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineAlbaranDelete.as_view(), name='CDNX_invoicing_linealbaranpurchasess_sublist_delete'),

    url(r'^tickets$', TicketList.as_view(), name='CDNX_invoicing_ticketpurchasess_list'),
    url(r'^tickets/add$', TicketCreate.as_view(), name='CDNX_invoicing_ticketpurchasess_add'),
    url(r'^tickets/addmodal$', TicketCreateModal.as_view(), name='CDNX_invoicing_ticketpurchasess_addmodal'),
    url(r'^tickets/(?P<pk>\w+)$', TicketDetails.as_view(), name='CDNX_invoicing_ticketpurchasess_details'),
    url(r'^tickets/(?P<pk>\w+)/edit$', TicketUpdate.as_view(), name='CDNX_invoicing_ticketpurchasess_edit'),
    url(r'^tickets/(?P<pk>\w+)/editmodal$', TicketUpdateModal.as_view(), name='CDNX_invoicing_ticketpurchasess_editmodal'),
    url(r'^tickets/(?P<pk>\w+)/delete$', TicketDelete.as_view(), name='CDNX_invoicing_ticketpurchasess_delete'),
    url(r'^tickets/(?P<pk>\w+)/print$', TicketPrint.as_view(), name='CDNX_invoicing_ticketpurchasess_print'),

    url(r'^linetickets$', LineTicketList.as_view(), name='CDNX_invoicing_lineticketpurchasess_list'),
    url(r'^linetickets/(?P<pk>\w+)/delete$', LineTicketDelete.as_view(), name='CDNX_invoicing_lineticketpurchasess_delete'),
    url(r'^linetickets/(?P<pk>\w+)/sublist$', LineTicketSubList.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist'),
    url(r'^linetickets/(?P<pk>\w+)/sublist/add$', LineTicketCreateModal.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_add'),
    url(r'^linetickets/(?P<pk>\w+)/sublist/addmodal$', LineTicketCreateModal.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_addmodal'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineTicketDetailsModal.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_details'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineTicketUpdateModal.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_edit'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineTicketUpdateModal.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_editmodal'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineTicketDelete.as_view(), name='CDNX_invoicing_lineticketpurchasess_sublist_delete'),

    url(r'^ticketrectifications$', TicketRectificationList.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_list'),
    url(r'^ticketrectifications/add$', TicketRectificationCreate.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_add'),
    url(r'^ticketrectifications/addmodal$', TicketRectificationCreateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_addmodal'),
    url(r'^ticketrectifications/(?P<pk>\w+)$', TicketRectificationDetails.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_details'),
    url(r'^ticketrectifications/(?P<pk>\w+)/edit$', TicketRectificationUpdate.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_edit'),
    url(r'^ticketrectifications/(?P<pk>\w+)/editmodal$', TicketRectificationUpdateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_editmodal'),
    url(r'^ticketrectifications/(?P<pk>\w+)/delete$', TicketRectificationDelete.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_delete'),
    url(r'^ticketrectifications/(?P<pk>\w+)/print$', TicketRectificationPrint.as_view(), name='CDNX_invoicing_ticketrectificationpurchasess_print'),

    url(r'^lineticketrectifications$', LineTicketRectificationList.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_list'),
    url(r'^lineticketrectifications/(?P<pk>\w+)/delete$', LineTicketRectificationDelete.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_delete'),
    url(r'^lineticketrectifications/(?P<pk>\w+)/sublist$', LineTicketRectificationSubList.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist'),
    url(r'^lineticketrectifications/(?P<pk>\w+)/sublist/add$', LineTicketRectificationCreateModal.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_add'),
    url(r'^lineticketrectifications/(?P<pk>\w+)/sublist/addmodal$', LineTicketRectificationCreateModal.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_addmodal'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineTicketRectificationDetailsModal.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_details'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineTicketRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_edit'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineTicketRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_editmodal'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineTicketRectificationDelete.as_view(), name='CDNX_invoicing_lineticketrectificationpurchasess_sublist_delete'),

    url(r'^invoices$', InvoiceList.as_view(), name='CDNX_invoicing_invoicepurchasess_list'),
    url(r'^invoices/add$', InvoiceCreate.as_view(), name='CDNX_invoicing_invoicepurchasess_add'),
    url(r'^invoices/addmodal$', InvoiceCreateModal.as_view(), name='CDNX_invoicing_invoicepurchasess_addmodal'),
    url(r'^invoices/(?P<pk>\w+)$', InvoiceDetails.as_view(), name='CDNX_invoicing_invoicepurchasess_details'),
    url(r'^invoices/(?P<pk>\w+)/edit$', InvoiceUpdate.as_view(), name='CDNX_invoicing_invoicepurchasess_edit'),
    url(r'^invoices/(?P<pk>\w+)/editmodal$', InvoiceUpdateModal.as_view(), name='CDNX_invoicing_invoicepurchasess_editmodal'),
    url(r'^invoices/(?P<pk>\w+)/delete$', InvoiceDelete.as_view(), name='CDNX_invoicing_invoicepurchasess_delete'),
    url(r'^invoices/(?P<pk>\w+)/print$', InvoicePrint.as_view(), name='CDNX_invoicing_invoicepurchasess_print'),

    url(r'^lineinvoices$', LineInvoiceList.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_list'),
    url(r'^lineinvoices/(?P<pk>\w+)/delete$', LineInvoiceDelete.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_delete'),
    url(r'^lineinvoices/(?P<pk>\w+)/sublist$', LineInvoiceSubList.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist'),
    url(r'^lineinvoices/(?P<pk>\w+)/sublist/add$', LineInvoiceCreateModal.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_add'),
    url(r'^lineinvoices/(?P<pk>\w+)/sublist/addmodal$', LineInvoiceCreateModal.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_addmodal'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineInvoiceDetailsModal.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_details'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineInvoiceUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_edit'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineInvoiceUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_editmodal'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineInvoiceDelete.as_view(), name='CDNX_invoicing_lineinvoicepurchasess_sublist_delete'),

    url(r'^invoicerectifications$', InvoiceRectificationList.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_list'),
    url(r'^invoicerectifications/add$', InvoiceRectificationCreate.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_add'),
    url(r'^invoicerectifications/addmodal$', InvoiceRectificationCreateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_addmodal'),
    url(r'^invoicerectifications/(?P<pk>\w+)$', InvoiceRectificationDetails.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_details'),
    url(r'^invoicerectifications/(?P<pk>\w+)/edit$', InvoiceRectificationUpdate.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_edit'),
    url(r'^invoicerectifications/(?P<pk>\w+)/editmodal$', InvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_editmodal'),
    url(r'^invoicerectifications/(?P<pk>\w+)/delete$', InvoiceRectificationDelete.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_delete'),
    url(r'^invoicerectifications/(?P<pk>\w+)/print$', InvoiceRectificationPrint.as_view(), name='CDNX_invoicing_invoicerectificationpurchasess_print'),

    url(r'^lineinvoicerectifications$', LineInvoiceRectificationList.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_list'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/editmodal$', LineInvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_editmodal'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/delete$', LineInvoiceRectificationDelete.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_delete'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/sublist$', LineInvoiceRectificationSubList.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/sublist/add$', LineInvoiceRectificationCreateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_add'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/sublist/addmodal$', LineInvoiceRectificationCreateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_addmodal'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineInvoiceRectificationDetailsModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_details'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', LineInvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_edit'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineInvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_editmodal'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineInvoiceRectificationDelete.as_view(), name='CDNX_invoicing_lineinvoicerectificationpurchasess_sublist_delete'),

    url(r'^budgetdocuments$', BudgetDocumentList.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_list'),
    url(r'^budgetdocuments/(?P<pk>\w+)$', BudgetDocumentDetails.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_details'),
    url(r'^budgetdocuments/(?P<pk>\w+)/delete$', BudgetDocumentDelete.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_delete'),
    url(r'^budgetdocuments/(?P<pk>\w+)/sublist$', BudgetDocumentSubList.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist'),
    url(r'^budgetdocuments/(?P<pk>\w+)/sublist/add$', BudgetDocumentCreateModal.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_add'),
    url(r'^budgetdocuments/(?P<pk>\w+)/sublist/addmodal$', BudgetDocumentCreateModal.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_addmodal'),
    url(r'^budgetdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', BudgetDocumentDetailsModal.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_details'),
    url(r'^budgetdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', BudgetDocumentUpdateModal.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_edit'),
    url(r'^budgetdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edimodalt$', BudgetDocumentUpdateModal.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_editmodal'),
    url(r'^budgetdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', BudgetDocumentDelete.as_view(), name='CDNX_invoicing_budgetpurchasesdocuments_sublist_delete'),

    url(r'^orderdocuments$', OrderDocumentList.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_list'),
    url(r'^orderdocuments/(?P<pk>\w+)$', OrderDocumentDetails.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_details'),
    url(r'^orderdocuments/(?P<pk>\w+)/delete$', OrderDocumentDelete.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_delete'),
    url(r'^orderdocuments/(?P<pk>\w+)/sublist$', OrderDocumentSubList.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist'),
    url(r'^orderdocuments/(?P<pk>\w+)/sublist/add$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_add'),
    url(r'^orderdocuments/(?P<pk>\w+)/sublist/addmodal$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_addmodal'),
    url(r'^orderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', OrderDocumentDetailsModal.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_details'),
    url(r'^orderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_edit'),
    url(r'^orderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_editmodal'),
    url(r'^orderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', OrderDocumentDelete.as_view(), name='CDNX_invoicing_orderpurchasesdocuments_sublist_delete'),

    url(r'^albarandocuments$', AlbaranDocumentList.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_list'),
    url(r'^albarandocuments/(?P<pk>\w+)$', AlbaranDocumentDetails.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_details'),
    url(r'^albarandocuments/(?P<pk>\w+)/delete$', AlbaranDocumentDelete.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_delete'),
    url(r'^albarandocuments/(?P<pk>\w+)/sublist$', AlbaranDocumentSubList.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist'),
    url(r'^albarandocuments/(?P<pk>\w+)/sublist/add$', AlbaranDocumentCreateModal.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_add'),
    url(r'^albarandocuments/(?P<pk>\w+)/sublist/addmodal$', AlbaranDocumentCreateModal.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_addmodal'),
    url(r'^albarandocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', AlbaranDocumentDetailsModal.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_details'),
    url(r'^albarandocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', AlbaranDocumentUpdateModal.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_edit'),
    url(r'^albarandocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', AlbaranDocumentUpdateModal.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_editmodal'),
    url(r'^albarandocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', AlbaranDocumentDelete.as_view(), name='CDNX_invoicing_albaranpurchasesdocuments_sublist_delete'),

    url(r'^ticketdocuments$', TicketDocumentList.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_list'),
    url(r'^ticketdocuments/(?P<pk>\w+)$', TicketDocumentDetails.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_details'),
    url(r'^ticketdocuments/(?P<pk>\w+)/delete$', TicketDocumentDelete.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_delete'),
    url(r'^ticketdocuments/(?P<pk>\w+)/sublist$', TicketDocumentSubList.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist'),
    url(r'^ticketdocuments/(?P<pk>\w+)/sublist/add$', TicketDocumentCreateModal.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_add'),
    url(r'^ticketdocuments/(?P<pk>\w+)/sublist/addmodal$', TicketDocumentCreateModal.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_addmodal'),
    url(r'^ticketdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', TicketDocumentDetailsModal.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_details'),
    url(r'^ticketdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', TicketDocumentUpdateModal.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_edit'),
    url(r'^ticketdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', TicketDocumentUpdateModal.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_editmodal'),
    url(r'^ticketdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', TicketDocumentDelete.as_view(), name='CDNX_invoicing_ticketpurchasesdocuments_sublist_delete'),

    url(r'^ticketrectificationdocuments$', TicketRectificationDocumentList.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_list'),
    url(r'^ticketrectificationdocuments/(?P<pk>\w+)$', TicketRectificationDocumentDetails.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_details'),
    url(r'^ticketrectificationdocuments/(?P<pk>\w+)/delete$', TicketRectificationDocumentDelete.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_delete'),
    url(r'^ticketrectificationdocuments/(?P<pk>\w+)/sublist$', TicketRectificationDocumentSubList.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist'),
    url(r'^ticketrectificationdocuments/(?P<pk>\w+)/sublist/add$', TicketRectificationDocumentCreateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_add'),
    url(r'^ticketrectificationdocuments/(?P<pk>\w+)/sublist/addmodal$', TicketRectificationDocumentCreateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_addmodal'),
    url(r'^ticketrectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', TicketRectificationDocumentDetailsModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_details'),
    url(r'^ticketrectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', TicketRectificationDocumentUpdateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_edit'),
    url(r'^ticketrectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', TicketRectificationDocumentUpdateModal.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_editmodal'),
    url(r'^ticketrectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', TicketRectificationDocumentDelete.as_view(), name='CDNX_invoicing_ticketrectificationpurchasesdocuments_sublist_delete'),

    url(r'^invoicedocuments$', InvoiceDocumentList.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_list'),
    url(r'^invoicedocuments/(?P<pk>\w+)$', InvoiceDocumentDetails.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_details'),
    url(r'^invoicedocuments/(?P<pk>\w+)/delete$', InvoiceDocumentDelete.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_delete'),
    url(r'^invoicedocuments/(?P<pk>\w+)/sublist$', InvoiceDocumentSubList.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist'),
    url(r'^invoicedocuments/(?P<pk>\w+)/sublist/add$', InvoiceDocumentCreateModal.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_add'),
    url(r'^invoicedocuments/(?P<pk>\w+)/sublist/addmodal$', InvoiceDocumentCreateModal.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_addmodal'),
    url(r'^invoicedocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', InvoiceDocumentDetailsModal.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_details'),
    url(r'^invoicedocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', InvoiceDocumentUpdateModal.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_edit'),
    url(r'^invoicedocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', InvoiceDocumentUpdateModal.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_editmodal'),
    url(r'^invoicedocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', InvoiceDocumentDelete.as_view(), name='CDNX_invoicing_invoicepurchasesdocuments_sublist_delete'),

    url(r'^invoicerectificationdocuments$', InvoiceRectificationDocumentList.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_list'),
    url(r'^invoicerectificationdocuments/(?P<pk>\w+)$', InvoiceRectificationDocumentDetails.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_details'),
    url(r'^invoicerectificationdocuments/(?P<pk>\w+)/delete$', InvoiceRectificationDocumentDelete.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_delete'),
    url(r'^invoicerectificationdocuments/(?P<pk>\w+)/sublist$', InvoiceRectificationDocumentSubList.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist'),
    url(r'^invoicerectificationdocuments/(?P<pk>\w+)/sublist/add$', InvoiceRectificationDocumentCreateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_add'),
    url(r'^invoicerectificationdocuments/(?P<pk>\w+)/sublist/addmodal$', InvoiceRectificationDocumentCreateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_addmodal'),
    url(r'^invoicerectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', InvoiceRectificationDocumentDetailsModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_details'),
    url(r'^invoicerectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', InvoiceRectificationDocumentUpdateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_edit'),
    url(r'^invoicerectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', InvoiceRectificationDocumentUpdateModal.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_editmodal'),
    url(r'^invoicerectificationdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', InvoiceRectificationDocumentDelete.as_view(), name='CDNX_invoicing_invoicerectificationpurchasesdocuments_sublist_delete'),

    url(r'^productfinals/foreignpurchases/(?P<search>[\w\W]+|\*)$', ProductFinalOfBudgetPurchase.as_view(), name='CDNX_invoicing_purchase_product_budget'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-27 07:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0011_auto_20180202_0826'),
        ('codenerix_invoicing', '0013_salesorderdocument_removed'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='purchaseslinealbaran',
            name='product_unique',
        ),
        migrations.AddField(
            model_name='purchaseslinealbaran',
            name='product_unique',
            field=models.ManyToManyField(related_name='line_albaran_purchases', to='codenerix_products.ProductUnique', verbose_name='Product Unique'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-30 12:47
from __future__ import unicode_literals

import codenerix.fields
import codenerix.lib.helpers
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0008_auto_20180130_1312'),
    ]

    operations = [
        migrations.CreateModel(
            name='SalesOrderDocument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('doc_path', codenerix.fields.FileAngularField(max_length=200, upload_to=codenerix.lib.helpers.upload_path, verbose_name='Doc Path')),
                ('name_file', models.CharField(max_length=254, verbose_name='Name')),
                ('notes', models.TextField(blank=True, max_length=256, null=True, verbose_name='Notes')),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_document_sales', to='codenerix_invoicing.TypeDocument', verbose_name='Document type')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='order_document_sales', to='codenerix_invoicing.SalesOrder', verbose_name='Sales order')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from decimal import Decimal

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.urls import reverse
from django.db.models import F, Sum
from django.utils.translation import gettext as _

from codenerix_invoicing.models import Haulier
# from codenerix_invoicing.models_sales import SalesBasket, SalesLines__________staticmethod as SalesLineBasket, ROLE_BASKET_SHOPPINGCART
from codenerix_products.models import ProductFinal


class ShoppingCartProxy(object):
    pass
"""
class ShoppingCartProxy_XXXX(object):
    __slots__ = ['_lang', '_cart', '_session', '_apply_surcharge', '_lines', '_quantities', '_products', '_totals', '__price_base', '__price_tax', '__ws_line', '__wos_line', ]
    SESSION_KEY = 'cdnx-shopping-cart'

    def __init__(self, request):
        self._lang = None
        for x in settings.LANGUAGES:
            if x[0] == request.LANGUAGE_CODE:
                self._lang = request.LANGUAGE_CODE.lower()
                break
        else:
            self._lang = settings.LANGUAGES[0][0].lower()

        self._cart = None
        self._session = None
        self._apply_surcharge = False

        self.__price_base = None
        self.__price_tax = None
        self.__ws_line = None
        self.__wos_line = None

        if request.user.is_authenticated:
            customer = request.user.person.customer
            if customer:
                self._apply_surcharge = customer.apply_equivalence_surcharge
            else:
                raise Exception(_('User {user} is not a Customer').format(user=request.user.username))

            try:
                self._cart = SalesBasket.objects.get(customer=customer, role=ROLE_BASKET_SHOPPINGCART)
            except ObjectDoesNotExist:
                pos = None
                self._cart = SalesBasket(customer=customer, role=ROLE_BASKET_SHOPPINGCART, pos_slot=pos)
                self._cart.save()

            if hasattr(request.body, 'transport'):
                transport = Haulier.objects.filter(name=request.body.transport).first()
                if transport is None:
                    transport = Haulier()
                    transport.name = request.body.transport
                    transport.save()

                self._cart.haulier = transport
                self._cart.save()

            # Merge anonymous to user cart
            if ShoppingCartProxy.SESSION_KEY in request.session:
                for product_dict in request.session[ShoppingCartProxy.SESSION_KEY]:
                    try:
                        product = ProductFinal.objects.get(pk=product_dict['pk'])
                    except ObjectDoesNotExist:
                        product = None
                    if product:
                        try:
                            line = SalesLineBasket.objects.get(basket=self._cart, product=product, removed=False)
                            line.quantity += product_dict['quantity']
                            line.save()
                        except ObjectDoesNotExist:
                            line = SalesLineBasket(
                                basket=self._cart,
                                product=product,
                                quantity=product_dict['quantity'],
                                price_base=product.price_base,
                                code=product.code
                            )
                            line.save()
                del request.session[ShoppingCartProxy.SESSION_KEY]
                request.session.modified = True
        else:
            if ShoppingCartProxy.SESSION_KEY not in request.session:
                request.session[ShoppingCartProxy.SESSION_KEY] = []
                request.session.modified = True

            self._session = request.session

        self._lines = None
        self._quantities = None
        self._products = None
        self._totals = None

    @property
    def lines(self):
        if self._lines is None:
            self._quantities = {}
            if self._cart is not None:
                self._lines = self._cart.line_basket_sales.filter(removed=False).values(
                    'product__pk',
                    'quantity'
                ).annotate(
                    pk=F('product__pk')
                )

                for line in self._lines:
                    line.pop('product__pk')
                    self._quantities[line['pk']] = line['quantity']

            elif self._session is not None:
                self._lines = self._session[ShoppingCartProxy.SESSION_KEY]
                self._quantities = {line['pk']: line['quantity'] for line in self._lines}

        return self._lines

    @property
    def products(self):
        return self.get_products()

    def get_products(self, onlypublic=None):
        if self._products is None:
            products = ProductFinal.objects.filter(
                pk__in=[int(line['pk']) for line in self.lines]
            )
            if onlypublic:
                products = products.filter(product__public=True)
            products = products.select_related(
                'product',
            ).only(
                'pk',
                'price',
                'stock_real',
                '{}__name'.format(self._lang),
                '{}__slug'.format(self._lang),
                '{}__description_short'.format(self._lang),
                'product__id',
                'product__code',
                'weight',
                'product__weight',
                'product__force_stock',
                'code'
            ).annotate(
                product_pk=F('product__id'),
                name=F('{}__name'.format(self._lang)),
                slug=F('{}__slug'.format(self._lang)),
                description=F('{}__description_short'.format(self._lang)),
                code_product=F('product__code'),
                force_stock=F('product__force_stock')
            )

            self._products = {
                'count': 0,
                'subtotal': Decimal("0"),
                'total': Decimal("0"),
                'tax': Decimal("0"),
                'products': [],
            }

            for final_product in products:

                features = final_product.product.product_features.all().values(
                    'feature__{}__description'.format(self._lang),
                    'value'
                ).annotate(
                    name=F('feature__{}__description'.format(self._lang))
                )

                attributes = final_product.products_final_attr.all().values(
                    'attribute__{}__description'.format(self._lang),
                    'value'
                ).annotate(
                    name=F('attribute__{}__description'.format(self._lang))
                )
                price = final_product.calculate_price()

                stock_locked = final_product.line_basket_sales.filter(
                    basket__expiration_date__isnull=False
                ).aggregate(
                    quantity=Sum('quantity')
                )['quantity'] or 0

                url_reverse = reverse('sluglevel_get', kwargs={'slug1': final_product.slug, })
                if final_product.code is None:
                    code = final_product.code_product
                else:
                    code = final_product.code
                img = final_product.product.products_image.filter(principal=True).first()
                if img:
                    thumbnail = img.image.url
                else:
                    thumbnail = ''
                if final_product.weight:
                    weight = final_product.weight
                else:
                    weight = final_product.product.weight
                if weight is None:
                    weight = 0
                self._products['products'].append({
                    'pk': final_product.pk,
                    'name': final_product.name,
                    'code': code,
                    'code_product': final_product.code_product,
                    'description': final_product.description,
                    'url': url_reverse,
                    'thumbnail': thumbnail,
                    'quantity': self._quantities[final_product.pk],
                    'stock_real': final_product.stock_real - stock_locked,
                    'force_stock': int(final_product.force_stock),
                    'tax': Decimal(self._quantities[final_product.pk]) * price['tax'],
                    'base_price': price['price_base'],
                    'unit_price': price['price_total'],
                    'weight': weight,
                    # by compatibility
                    'overcharge': Decimal("0"),
                    'total_price': Decimal(self._quantities[final_product.pk]) * price['price_total'],
                    'features': [{
                        'name': feature['name'],
                        'value': feature['value']
                    } for feature in features],
                    'attributes': [{
                        'name': attribute['name'],
                        'value': attribute['value'],
                    } for attribute in attributes]
                })

                self._products['count'] += 1
                self._products['subtotal'] += Decimal(self._quantities[final_product.pk]) * price['price_base']
                self._products['total'] += Decimal(self._quantities[final_product.pk]) * price['price_base'] + self._products['products'][-1]['tax']
                self._products['tax'] += self._products['products'][-1]['tax']

        return self._products

    @property
    def totals(self):
        if self._totals is None:
            products = ProductFinal.objects.filter(
                pk__in=[int(line['pk']) for line in self.lines]
            ).select_related(
                'product',
            ).only(
                'pk',
                'price',
                'product__tax__tax'
            )

            self._totals = {
                'count': 0,
                'subtotal': Decimal("0"),
                'total': Decimal("0"),
                'tax': Decimal("0"),
            }

            for final_product in products:
                price = final_product.calculate_price()
                self._totals['count'] += 1
                self._totals['subtotal'] += Decimal(self._quantities[final_product.pk]) * price['price_base']
                self._totals['total'] += Decimal(self._quantities[final_product.pk]) * price['price_total']
                self._totals['tax'] += Decimal(self._quantities[final_product.pk]) * price['tax']

        return self._totals

    @property
    def user_cart(self):
        return self._cart

    def set_address(self, address_invoice, address_delivery):
        self._cart.address_invoice = address_invoice.address_invoice
        self._cart.address_delivery = address_delivery.address_delivery
        self._cart.save()

    def product(self, product_pk):
        product = ProductFinal.objects.get(pk=product_pk)
        price = product.calculate_price()

        result = self.totals.copy()
        result['total_price'] = Decimal(self._quantities[product_pk]) * price['price_total']

        return result

    def get_info_prices(self, onlypublic=False):
        if not self.__price_tax or not self.__price_base:

            with_stock = []
            without_stock = []

            price_base = Decimal("0")
            price_tax = Decimal("0")
            ws_line = {}
            wos_line = None

            for line in self.get_products(onlypublic=onlypublic)['products']:
                ws_line = {
                    'description': line['name'],
                    'price': line['unit_price'] * Decimal(line['quantity']),
                    'quantity': line['quantity'],
                }
                wos_line = {
                    'description': line['name'],
                    'price': line['base_price'],
                    'quantity': 0.0
                }

                if line['force_stock']:
                    if line['stock_real'] >= line['quantity']:
                        wos_line = None
                    else:
                        if line['stock_real'] == 0:
                            ws_line = None
                        wos_line['quantity'] = line['quantity'] - line['stock_real']
                else:
                    wos_line = None

                if ws_line:
                    with_stock.append(ws_line)
                    price_base += line['base_price'] * Decimal(line['quantity'])
                    price_tax += line['tax'] - line['overcharge']
                if wos_line:
                    without_stock.append(wos_line)

            if without_stock:
                without_stock += with_stock
                with_stock = []

            prices = {
                'price_base': price_base,
                'price_tax': price_tax,
                'price_total': price_tax + price_base,
                'products_with_stock': with_stock,
                'products_without_stock': without_stock,
            }
        else:
            prices = {
                'price_base': self.__price_base,
                'price_tax': self.__price_tax,
                'price_total': self.__price_tax + self.__price_base,
                'ws_line': self.__ws_line,
                'wos_line': self.__wos_line,
            }
        return prices

    def _reset_data(self):
        self._lines = None
        self._quantities = None
        self._products = None
        self._totals = None
        # info prices
        self.__price_base = None
        self.__price_tax = None
        self.__ws_line = None
        self.__wos_line = None

    def _set_product(self, product_pk, quantity, add=False):
        product = ProductFinal.objects.get(pk=product_pk)
        if self._cart is not None:
            try:
                line = SalesLineBasket.objects.get(basket=self._cart, product=product)  # , removed=False)
                if line.removed:
                    line.removed = False
                    line.quantity = 0
                line.quantity = line.quantity + quantity if add else quantity
            except ObjectDoesNotExist:
                line = SalesLineBasket(basket=self._cart, product=product, code=product.code)
                line.quantity = quantity
            finally:
                prices = product.calculate_price()
                line.price = prices['price_total']
                line.price_base = prices['price_base']
                line.save()
        elif self._session is not None:
            for product_dict in self._session[ShoppingCartProxy.SESSION_KEY]:
                if product_dict['pk'] == product.pk:
                    product_dict['quantity'] = product_dict['quantity'] + quantity if add else quantity
                    break
            else:
                self._session[ShoppingCartProxy.SESSION_KEY].append({
                    'pk': product_pk,
                    'quantity': quantity
                })
            self._session.modified = True

    def add(self, product_pk, quantity):
        self._set_product(product_pk, quantity, add=True)
        self._reset_data()

    def edit(self, product_pk, quantity):
        self._set_product(product_pk, quantity)
        self._reset_data()

    def remove(self, product_pk):
        product = ProductFinal.objects.get(pk=product_pk)
        if self._cart is not None:
            line = SalesLineBasket.objects.get(basket=self._cart, product=product)
            line.delete()
        elif self._session is not None:
            if ShoppingCartProxy.SESSION_KEY in self._session:
                for product in self._session[ShoppingCartProxy.SESSION_KEY]:
                    if product['pk'] == product_pk:
                        self._session[ShoppingCartProxy.SESSION_KEY].remove(product)
                        self._session.modified = True
        self._reset_data()

    def list(self):
        return SalesLineBasket.objects.filter(basket=self._cart)
"""



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-05-15 10:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0018_salesalbaran_send'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleslines',
            name='discounts_basket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Discounts'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='discounts_invoice',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Discounts'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='discounts_order',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Discounts'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='discounts_ticket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Discounts'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='equivalence_surcharges_basket',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, null=True, verbose_name='Equivalence surcharge'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='equivalence_surcharges_invoice',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, null=True, verbose_name='Equivalence surcharge'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='equivalence_surcharges_order',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, null=True, verbose_name='Equivalence surcharge'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='equivalence_surcharges_ticket',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, max_digits=10, null=True, verbose_name='Equivalence surcharge'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='subtotal_basket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='subtotal_invoice',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='subtotal_order',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='subtotal_ticket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Subtotal'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='taxes_basket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Taxes'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='taxes_invoice',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Taxes'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='taxes_order',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Taxes'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='taxes_ticket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Taxes'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='total_basket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='total_invoice',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='total_order',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='total_ticket',
            field=models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=10, verbose_name='Total'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext as _
from django.conf import settings

from codenerix.forms import GenModelForm
from codenerix.widgets import WysiwygAngularInput

from codenerix_invoicing.models import BillingSeries, LegalNote, TypeDocument, MODELS, Haulier  # , ProductStock, StockMovement, StockMovementProduct


class BillingSeriesForm(GenModelForm):
    class Meta:
        model = BillingSeries
        exclude = []
        widgets = {
            'description': WysiwygAngularInput(),
            'observations': WysiwygAngularInput(),
        }

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['code', 6],
                ['default', 6],
                ['description', 6],
                ['observations', 6],)
        ]
        return g


class LegalNoteForm(GenModelForm):
    class Meta:
        model = LegalNote
        exclude = []
        widgets = {
            'legal_note': WysiwygAngularInput()
        }

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['legal_note', 6],
                ['public', 6],)
        ]
        return g


class TypeDocumentForm(GenModelForm):
    class Meta:
        model = TypeDocument
        exclude = []

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['code', 6],)
        ]
        return g


# MODELS
for info in MODELS:
    field = info[0]
    model = info[1]
    for lang_code in settings.LANGUAGES_DATABASES:
        exec("from codenerix_invoicing.models import {}Text{}\n".format(model, lang_code))
        query = """
class {model}TextForm{lang}(GenModelForm):\n
    class Meta:\n
        model={model}Text{lang}\n
        exclude = []\n
        widgets = {{\n
            'description': WysiwygAngularInput(),\n
        }}\n
    def __groups__(self):\n
        return [(_('Details'),12,"""
        if lang_code == settings.LANGUAGES_DATABASES[0]:
            query += """
                ['name', 12, None, None, None, None, None, ["ng-blur=refresh_lang_field('name', '{model}TextForm', [{languages}])"]],
                ['description', 12, None, None, None, None, None, ["ng-blur=refresh_lang_field('description', '{model}TextForm', [{languages}])"]],
            )]\n"""
        else:
            query += """
                ['name', 12],
                ['description', 12],
            )]\n"""

        exec(query.format(model=model, lang=lang_code, languages="'{}'".format("','".join(settings.LANGUAGES_DATABASES))))

"""
class ProductStockForm(GenModelForm):
    class Meta:
        model = ProductStock
        exclude = []

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['batch', 6],
                ['product_final', 6],
                ['quantity', 6],)
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (_('Details'), 12,
                ['batch', 6],
                ['product_final', 6],
                ['quantity', 6],)
        ]
        return g


class ProductStockOwnForm(GenModelForm):
    class Meta:
        model = ProductStock
        exclude = ['batch']

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['product_final', 6],
                ['quantity', 6],)
        ]
        return g


class StockMovementForm(GenModelForm):
    class Meta:
        model = StockMovement
        exclude = []

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['batch_source', 6],
                ['batch_destination', 6],
                ['status', 6],)
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (_('Details'), 12,
                ['batch_source', 6],
                ['batch_destination', 6],
                ['status', 6],)
        ]
        return g


class StockMovementProductForm(GenModelForm):
    class Meta:
        model = StockMovementProduct
        exclude = ['stock_movement', ]

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['product_final', 6],
                ['quantity', 6],)
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (_('Details'), 12,
                ['product_final', 6],
                ['quantity', 6],)
        ]
        return g
"""

class HaulierForm(GenModelForm):
    class Meta:
        model = Haulier
        exclude = ['name_file', ]

    def __groups__(self):
        g = [
            (_('Details'), 12,
                ['name', 6],
                ['image', 6],
                ['url_public', 6],
                ['url_tracking', 6],)
        ]
        return g



# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.db.models import Q
from django.utils.translation import gettext as _

from codenerix.views import GenList, GenCreate, GenCreateModal, GenUpdate, GenUpdateModal, GenDelete, GenDetail, GenDetailModal

from codenerix_invoicing.models_sales import ReasonModification
from codenerix_invoicing.models_sales import ReasonModificationLineOrder
from codenerix_invoicing.models_sales import ReasonModificationLineBasket, ReasonModificationLineAlbaran, ReasonModificationLineTicket, ReasonModificationLineTicketRectification, ReasonModificationLineInvoice, ReasonModificationLineInvoiceRectification
from codenerix_invoicing.forms_reason import ReasonModificationForm
from codenerix_invoicing.forms_reason import ReasonModificationLineBasketForm, ReasonModificationLineOrderForm, ReasonModificationLineAlbaranForm, ReasonModificationLineTicketForm, ReasonModificationLineTicketRectificationForm, ReasonModificationLineInvoiceForm, ReasonModificationLineInvoiceRectificationForm


# ###########################################
# ReasonModification
class ReasonModificationList(GenList):
    model = ReasonModification
    extra_context = {'menu': ['sales', 'ReasonModification'], 'bread': [_('Sales'), _('ReasonModification')]}


class ReasonModificationCreate(GenCreate):
    model = ReasonModification
    form_class = ReasonModificationForm


class ReasonModificationCreateModal(GenCreateModal, ReasonModificationCreate):
    pass


class ReasonModificationUpdate(GenUpdate):
    model = ReasonModification
    form_class = ReasonModificationForm


class ReasonModificationUpdateModal(GenUpdateModal, ReasonModificationUpdate):
    pass


class ReasonModificationDelete(GenDelete):
    model = ReasonModification


class ReasonModificationSubList(GenList):
    model = ReasonModification
    extra_context = {'menu': ['sales', 'ReasonModification'], 'bread': [_('Sales'), _('ReasonModification')]}


class ReasonModificationDetails(GenDetail):
    model = ReasonModification
    groups = ReasonModificationForm.__groups_details__()


class ReasonModificationDetailModal(GenDetailModal, ReasonModificationDetails):
    pass


# ###########################################
class ReasonModificationLineList(GenList):
    linkadd = False
    show_details = True


class ReasonModificationLineDetail(GenDetailModal, GenDetail):
    linkedit = False
    linkdelete = False


# ###########################################
# ReasonModificationLineBasket
class ReasonModificationLineBasketSubList(ReasonModificationLineList):
    model = ReasonModificationLineBasket

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(basket__pk=pk)
        return limit


class ReasonModificationLineBasketDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineBasket
    groups = ReasonModificationLineBasketForm.__groups_details__()


# ###########################################
# ReasonModificationLineOrder
class ReasonModificationLineOrderSubList(ReasonModificationLineList):
    model = ReasonModificationLineOrder

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(order__pk=pk)
        return limit


class ReasonModificationLineOrderDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineOrder
    groups = ReasonModificationLineOrderForm.__groups_details__()


# ###########################################
# ReasonModificationLineAlbaran
class ReasonModificationLineAlbaranSubList(ReasonModificationLineList):
    model = ReasonModificationLineAlbaran

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(albaran__pk=pk)
        return limit


class ReasonModificationLineAlbaranDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineAlbaran
    groups = ReasonModificationLineAlbaranForm.__groups_details__()


# ###########################################
# ReasonModificationLineTicket
class ReasonModificationLineTicketSubList(ReasonModificationLineList):
    model = ReasonModificationLineTicket

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(ticket__pk=pk)
        return limit


class ReasonModificationLineTicketDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineTicket
    groups = ReasonModificationLineTicketForm.__groups_details__()


# ###########################################
# ReasonModificationLineTicketRectification
class ReasonModificationLineTicketRectificationSubList(ReasonModificationLineList):
    model = ReasonModificationLineTicketRectification

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(ticket_rectification__pk=pk)
        return limit


class ReasonModificationLineTicketRectificationDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineTicketRectification
    groups = ReasonModificationLineTicketRectificationForm.__groups_details__()


# ###########################################
# ReasonModificationLineInvoice
class ReasonModificationLineInvoiceSubList(ReasonModificationLineList):
    model = ReasonModificationLineInvoice

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(invoice__pk=pk)
        return limit


class ReasonModificationLineInvoiceDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineInvoice
    groups = ReasonModificationLineInvoiceForm.__groups_details__()


# ###########################################
# ReasonModificationLineInvoiceRectification
class ReasonModificationLineInvoiceRectificationSubList(ReasonModificationLineList):
    model = ReasonModificationLineInvoiceRectification

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(invoice_rectification__pk=pk)
        return limit


class ReasonModificationLineInvoiceRectificationDetailModal(ReasonModificationLineDetail):
    model = ReasonModificationLineInvoiceRectification
    groups = ReasonModificationLineInvoiceRectificationForm.__groups_details__()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-05-15 11:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0019_auto_20180515_1235'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='saleslines',
            name='discounts',
        ),
        migrations.RemoveField(
            model_name='saleslines',
            name='equivalence_surcharges',
        ),
        migrations.RemoveField(
            model_name='saleslines',
            name='subtotal',
        ),
        migrations.RemoveField(
            model_name='saleslines',
            name='taxes',
        ),
        migrations.RemoveField(
            model_name='saleslines',
            name='total',
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-invoicing
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path as url
from .views_sales import CustomerCreate, CustomerCreateModal, CustomerDelete, CustomerList, CustomerUpdate, CustomerUpdateModal, CustomerDetails
from .views_sales import CustomerDocumentSubList, CustomerDocumentCreateModal, CustomerDocumentDetailsModal, CustomerDocumentUpdateModal, CustomerDocumentDelete
from .views_sales import CustomerForeignBudget, CustomerForeignShoppingCart

from .views_sales import BasketPassToOrder
from .views_sales import BasketPassToBudget
from .views_sales import BasketListSHOPPINGCART, BasketDetailsSHOPPINGCART, BasketCreateSHOPPINGCART, BasketCreateSHOPPINGCARTModal, BasketUpdateSHOPPINGCART, BasketDeleteSHOPPINGCART
from .views_sales import BasketListBUDGET, BasketDetailsBUDGET, BasketCreateBUDGET, BasketCreateBUDGETModal, BasketUpdateBUDGET, BasketDeleteBUDGET
from .views_sales import BasketListWISHLIST, BasketDetailsWISHLIST, BasketCreateWISHLIST, BasketCreateWISHLISTModal, BasketUpdateWISHLIST, BasketDeleteWISHLIST
from .views_sales import BasketForeignShoppingCart, BasketForeignBudget
from .views_sales import BasketPrintBUDGET

from .views_sales import LinesVending, LinesVendingEdit
from .views_sales import SalesLinesList, SalesLinesDetails, SalesLinesCreateWS

from .views_sales import OrderForeign
from .views_sales import OrderCreateAlbaran, OrderCreateTicket, OrderCreateInvoice, OrderStatus
from .views_sales import OrderCreate, OrderCreateModal, OrderDelete, OrderDetails, OrderList, OrderUpdate, OrderUpdateModal, OrderPrint
from .views_sales import OrderCreateModalFromBudget, OrderCreateModalFromShoppingCart

from .views_sales import OrderDocumentSubList, OrderDocumentCreateModal, OrderDocumentUpdateModal, OrderDocumentDelete

from .views_sales import AlbaranCreate, AlbaranCreateModal, AlbaranDelete, AlbaranList, AlbaranUpdate, AlbaranUpdateModal
from .views_sales import AlbaranDetails, AlbaranPrint, AlbaranSend
from .views_sales import AlbaranCreateTicket, AlbaranCreateInvoice

from .views_sales import InvoiceCreate, InvoiceCreateModal, InvoiceDelete, InvoiceList, InvoiceUpdate, InvoiceUpdateModal, InvoicePrint
from .views_sales import InvoiceDetails
from .views_sales import InvoiceCreateRectification, InvoiceCreateRectificationUnity
from .views_sales import InvoiceRectificationCreate, InvoiceRectificationCreateModal, InvoiceRectificationDelete, InvoiceRectificationList, InvoiceRectificationUpdate, InvoiceRectificationUpdateModal, InvoiceRectificationDetails, InvoiceRectificationPrint

from .views_sales import TicketCreate, TicketCreateModal, TicketDelete, TicketList, TicketUpdate, TicketUpdateModal
from .views_sales import TicketDetails, TicketPrint
from .views_sales import TicketCreateInvoice, TicketCreateRectification
from .views_sales import TicketRectificationCreate, TicketRectificationCreateModal, TicketRectificationDelete, TicketRectificationList, TicketRectificationUpdate, TicketRectificationUpdateModal, TicketRectificationDetails, TicketRectificationPrint


from .views_sales import PrintCounterDocumentBasketSublist, PrintCounterDocumentOrderSublist, PrintCounterDocumentAlbaranSublist, PrintCounterDocumentTicketSublist, PrintCounterDocumentTicketRectificationSublist, PrintCounterDocumentInvoiceSublist, PrintCounterDocumentInvoiceRectificationSublist

from .views_sales import LinesSubListBasket, LinesCreateModalBasket, LinesUpdateModalBasket
from .views_sales import LinesSubListOrder, LinesDetailModalOrder, LinesUpdateModalOrder
from .views_sales import LinesSubListAlbaran, LinesUpdateModalAlbaran
from .views_sales import LinesSubListInvoice, LinesUpdateModalInvoice, LinesDetailModalInvoice
from .views_sales import LinesSubListInvoiceRectification, LinesUpdateModalInvoiceRectification
from .views_sales import LinesSubListTicket, LinesUpdateModalTicket
from .views_sales import LinesSubListTicketRectification, LinesUpdateModalTicketRectification
from .views_sales import LineBasketDelete

from .views_sales import VendingPay, VendingPrint

from .views_reason import ReasonModificationList, ReasonModificationCreate, ReasonModificationCreateModal, ReasonModificationUpdate, ReasonModificationUpdateModal, ReasonModificationDelete, ReasonModificationSubList, ReasonModificationDetails, ReasonModificationDetailModal

from .views_reason import ReasonModificationLineTicketSubList, ReasonModificationLineTicketDetailModal
from .views_reason import ReasonModificationLineBasketSubList, ReasonModificationLineBasketDetailModal
from .views_reason import ReasonModificationLineOrderSubList, ReasonModificationLineOrderDetailModal
from .views_reason import ReasonModificationLineAlbaranSubList, ReasonModificationLineAlbaranDetailModal
from .views_reason import ReasonModificationLineTicketRectificationSubList, ReasonModificationLineTicketRectificationDetailModal
from .views_reason import ReasonModificationLineInvoiceSubList, ReasonModificationLineInvoiceDetailModal
from .views_reason import ReasonModificationLineInvoiceRectificationSubList, ReasonModificationLineInvoiceRectificationDetailModal
"""
  # , LinesDetailModalBasket, LinesDeleteBasket, LinesForeignBasket
from .views_sales import LineAlbaranCreate, LineAlbaranCreateModal, LineAlbaranDelete, LineAlbaranList, LineAlbaranUpdate, LineAlbaranUpdateModal
from .views_sales import LineInvoiceRectificationCreate, LineInvoiceRectificationCreateModal, LineInvoiceRectificationDelete, LineInvoiceRectificationList, LineInvoiceRectificationUpdate, LineInvoiceRectificationUpdateModal, LineInvoiceRectificationSubList, LineInvoiceRectificationDetailModal
from .views_sales import LineOrderCreateModal, LineOrderDelete, LineOrderDetailsModal, LineOrderList, LineOrderSubList, LineOrderUpdateModal
from .views_sales import LineInvoiceCreate, LineInvoiceCreateModal, LineInvoiceDelete, LineInvoiceList, LineInvoiceUpdate, LineInvoiceUpdateModal
from .views_sales import LineTicketRectificationCreate, LineTicketRectificationCreateModal, LineTicketRectificationDelete, LineTicketRectificationList, LineTicketRectificationUpdate, LineTicketRectificationUpdateModal, LineTicketRectificationSubList, LineTicketRectificationDetailModal
from .views_sales import LineTicketCreate, LineTicketCreateModal, LineTicketDelete, LineTicketList, LineTicketUpdate, LineTicketUpdateModal
, LineAlbaranDetailsModal, LineAlbaranSubList
, LineTicketSubList, LineTicketDetailsModal
, LineInvoiceSubList, LineInvoiceDetailsModal
, LineOrderForeign, LineOrderForeignCustom
, LineInvoiceForeign, LineTicketForeign

from .views_sales import LineBasketList, LineBasketCreate, LineBasketCreateModal, LineBasketUpdate, LineBasketUpdateModal, LineBasketDelete, LineBasketSubList, LineBasketDetails, LineBasketDetailModal, LineBasketForeign
from .views_sales import  LineBasketCreateModalPack

from .views_sales import ReservedProductList, ReservedProductCreate, ReservedProductUpdate, ReservedProductDelete
from .views_sales import BasketCreate, BasketCreateModal, BasketUpdate, BasketUpdateModal, BasketDetails, BasketDelete

from .views_sales import ShoppingCartManagement


from .views_sales import LineAlbaranForeign, LineTicketRectificationForeign, LineInvoiceRectificationForeign
from .views_sales import OrderDocumentList, OrderDocumentCreate, OrderDocumentUpdate,
, OrderDocumentDetails, OrderDocumentDetailModal
"""


urlpatterns = [
    url(r'^customers$', CustomerList.as_view(), name='CDNX_invoicing_customers_list'),
    url(r'^customers/add$', CustomerCreate.as_view(), name='CDNX_invoicing_customers_add'),
    url(r'^customers/addmodal$', CustomerCreateModal.as_view(), name='CDNX_invoicing_customers_addmodal'),
    url(r'^customers/(?P<pk>\w+)$', CustomerDetails.as_view(), name='CDNX_invoicing_customers_detail'),
    url(r'^customers/(?P<pk>\w+)/edit$', CustomerUpdate.as_view(), name='CDNX_invoicing_customers_edit'),
    url(r'^customers/(?P<pk>\w+)/editmodal$', CustomerUpdateModal.as_view(), name='CDNX_invoicing_customers_editmodal'),
    url(r'^customers/(?P<pk>\w+)/delete$', CustomerDelete.as_view(), name='CDNX_invoicing_customers_delete'),

    url(r'^customers/foreign/shoppingcarts/(?P<search>[\w\W]+|\*)$', CustomerForeignShoppingCart.as_view(), name='CDNX_invoicing_customers_foreign_from_shoppingcart'),
    url(r'^customers/foreign/(?P<search>[\w\W]+|\*)$', CustomerForeignBudget.as_view(), name='CDNX_invoicing_customers_foreign_from_budget'),

    url(r'^customerdocuments/(?P<pk>\w+)/sublist$', CustomerDocumentSubList.as_view(), name='CDNX_invoicing_customerdocuments_sublist'),
    url(r'^customerdocuments/(?P<pk>\w+)/sublist/add$', CustomerDocumentCreateModal.as_view(), name='CDNX_invoicing_customerdocuments_sublist_add'),
    url(r'^customerdocuments/(?P<pk>\w+)/sublist/addmodal$', CustomerDocumentCreateModal.as_view(), name='CDNX_invoicing_customerdocuments_sublist_addmodal'),
    url(r'^customerdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', CustomerDocumentDetailsModal.as_view(), name='CDNX_invoicing_customerdocuments_sublist_details'),
    url(r'^customerdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', CustomerDocumentUpdateModal.as_view(), name='CDNX_invoicing_customerdocuments_sublist_edit'),
    url(r'^customerdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', CustomerDocumentUpdateModal.as_view(), name='CDNX_invoicing_customerdocuments_sublist_editmodal'),
    url(r'^customerdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', CustomerDocumentDelete.as_view(), name='CDNX_invoicing_customerdocuments_sublist_delete'),

    url(r'^shoppingcarts$', BasketListSHOPPINGCART.as_view(), name='CDNX_invoicing_salesbaskets_shoppingcart_list'),
    url(r'^budgets$', BasketListBUDGET.as_view(), name='CDNX_invoicing_salesbaskets_budget_list'),
    url(r'^wishlists$', BasketListWISHLIST.as_view(), name='CDNX_invoicing_salesbaskets_wishlist_list'),

    url(r'^shoppingcarts/add$', BasketCreateSHOPPINGCART.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_add'),
    url(r'^budgets/add$', BasketCreateBUDGET.as_view(), name='CDNX_invoicing_salesbaskets_BUDGET_add'),
    url(r'^wishlists/add$', BasketCreateWISHLIST.as_view(), name='CDNX_invoicing_salesbaskets_WISHLIST_add'),


    # ################
    url(r'^vending/(?P<bpk>\w+)/(?P<pk>\w+)/editmodal$', LinesVendingEdit.as_view(), name='CDNX_invoicing_salesbaskets_shoppingcart_vending_edit'),
    url(r'^vending$', LinesVending.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_vending_details'),
    url(r'^vending/(?P<bpk>\w+)$', LinesVending.as_view(), name='CDNX_invoicing_salesbaskets_shoppingcart_vending_list'),
    url(r'^vending/(?P<pk>\w+)/pay$', VendingPay.as_view(), name='CDNX_invoicing_vending_pay'),
    url(r'^vending/(?P<pk>\w+)/print$', VendingPrint.as_view(), name='CDNX_invoicing_vending_print'),
    url(r'^vending/(?P<bpk>\w+)/(?P<pk>\w+)$', SalesLinesDetails.as_view(), name='CDNX_invoicing_saleslines_vending_details'),
    # ################


    url(r'^shoppingcarts/(?P<pk>\w+)$', BasketDetailsSHOPPINGCART.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_details'),
    url(r'^budgets/(?P<pk>\w+)$', BasketDetailsBUDGET.as_view(), name='CDNX_invoicing_salesbaskets_BUDGET_details'),
    url(r'^wishlists/(?P<pk>\w+)$', BasketDetailsWISHLIST.as_view(), name='CDNX_invoicing_salesbaskets_WISHLIST_details'),

    url(r'^shoppingcarts/addmodal$', BasketCreateSHOPPINGCARTModal.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_addmodal'),
    url(r'^budgets/addmodal$', BasketCreateBUDGETModal.as_view(), name='CDNX_invoicing_salesbaskets_BUDGET_addmodal'),
    url(r'^wishlists/addmodal$', BasketCreateWISHLISTModal.as_view(), name='CDNX_invoicing_salesbaskets_WISHLIST_addmodal'),

    url(r'^shoppingcarts/(?P<pk>\w+)/edit$', BasketUpdateSHOPPINGCART.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_edit'),
    url(r'^budgets/(?P<pk>\w+)/edit$', BasketUpdateBUDGET.as_view(), name='CDNX_invoicing_salesbaskets_BUDGET_edit'),
    url(r'^wishlists/(?P<pk>\w+)/edit$', BasketUpdateWISHLIST.as_view(), name='CDNX_invoicing_salesbaskets_WISHLIST_edit'),

    url(r'^shoppingcarts/(?P<pk>\w+)/delete$', BasketDeleteSHOPPINGCART.as_view(), name='CDNX_invoicing_salesbaskets_SHOPPINGCART_delete'),
    url(r'^budgets/(?P<pk>\w+)/delete$', BasketDeleteBUDGET.as_view(), name='CDNX_invoicing_salesbaskets_BUDGET_delete'),
    url(r'^wishlists/(?P<pk>\w+)/delete$', BasketDeleteWISHLIST.as_view(), name='CDNX_invoicing_salesbaskets_WISHLIST_delete'),

    url(r'^shoppingcarts/(?P<pk>\w+)/createbudget$', BasketPassToBudget.as_view(), name='CDNX_invoicing_salesbaskets_createbudget'),
    url(r'^budgets/(?P<pk>\w+)/createorder$', BasketPassToOrder.as_view(), name='CDNX_invoicing_salesbaskets_createorder'),

    url(r'^shoppingcarts/foreign/(?P<search>[\w\W]+|\*)$', BasketForeignShoppingCart.as_view(), name='CDNX_invoicing_salesbaskets_foreignkey_shoppingcart'),
    url(r'^budgets/foreign/(?P<search>[\w\W]+|\*)$', BasketForeignBudget.as_view(), name='CDNX_invoicing_salesbaskets_foreignkey_budget'),

    url(r'^budgets/(?P<pk>\w+)/print$', BasketPrintBUDGET.as_view(), name='CDNX_invoicing_budgetsaless_print'),

    # ################
    url(r'^lines$', SalesLinesList.as_view(), name='CDNX_invoicing_saleslines_list'),
    url(r'^salesliness$', SalesLinesList.as_view(), name='CDNX_invoicing_saleslines_list'),
    url(r'^lines/(?P<ipk>\w+)/addws$', SalesLinesCreateWS.as_view(), name='CDNX_invoicing_saleslines_addws'),

    url(r'^lines/(?P<pk>\w+)/basket_sublist$', LinesSubListBasket.as_view(), name='CDNX_invoicing_saleslines_sublist_basket'),
    url(r'^lines/(?P<pk>\w+)/basket_sublist/addmodal$', LinesCreateModalBasket.as_view(), name='CDNX_invoicing_saleslines_sublist_addmodal_basket'),
    url(r'^lines/(?P<cpk>\w+)/basket_sublist/(?P<pk>\w+)/editmodal$', LinesUpdateModalBasket.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_basket'),
    url(r'^lines/(?P<cpk>\w+)/basket_sublist/(?P<pk>\w+)/delete$', LineBasketDelete.as_view(), name='CDNX_invoicing_saleslinebs_sublist_delete'),

    # url(r'^lines/basket_foreign/(?P<search>[\w\W]+|\*)$', LinesForeignBasketOrder.as_view(), name='CDNX_invoicing_linessaless_foreign_for_order'),

    url(r'^lines/(?P<pk>\w+)/order_sublist$', LinesSubListOrder.as_view(), name='CDNX_invoicing_saleslines_sublist_order'),
    # url(r'^lines/(?P<pk>\w+)/order_sublist/addmodal$', LinesCreateModalOrder.as_view(), name='CDNX_invoicing_saleslines_sublist_addmodal_order'),
    url(r'^lines/(?P<cpk>\w+)/order_sublist/(?P<pk>\w+)/modal$', LinesDetailModalOrder.as_view(), name='CDNX_invoicing_saleslines_sublist_details_order'),
    url(r'^lines/(?P<cpk>\w+)/order_sublist/(?P<pk>\w+)/editmodal$', LinesUpdateModalOrder.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_order'),

    url(r'^lines/(?P<pk>\w+)/albaran_sublist$', LinesSubListAlbaran.as_view(), name='CDNX_invoicing_saleslines_sublist_albaran'),
    url(r'^lines/(?P<cpk>\w+)/albaran_sublist/(?P<pk>\w+)/editmodal$', LinesUpdateModalAlbaran.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_albaran'),

    url(r'^lines/(?P<pk>\w+)/invoice_sublist$', LinesSubListInvoice.as_view(), name='CDNX_invoicing_saleslines_sublist_invoice'),
    url(r'^lines/(?P<cpk>\w+)/invoice_sublist/(?P<pk>\w+)/modal$', LinesDetailModalInvoice.as_view(), name='CDNX_invoicing_saleslines_sublist_details_invoice'),
    url(r'^lines/(?P<cpk>\w+)/invoice_sublist/(?P<pk>\w+)/editmodal$', LinesUpdateModalInvoice.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_invoice'),

    url(r'^lines/(?P<pk>\w+)/invoicerectification_sublist$', LinesSubListInvoiceRectification.as_view(), name='CDNX_invoicing_saleslines_sublist_invoicerectification'),
    url(r'^lines/(?P<cpk>\w+)/invoicerectification_sublist/(?P<pk>\w+)/editmodal$', LinesUpdateModalInvoiceRectification.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_invoicerectification'),

    url(r'^lines/(?P<pk>\w+)/ticket_sublist$', LinesSubListTicket.as_view(), name='CDNX_invoicing_saleslines_sublist_ticket'),
    url(r'^lines/(?P<pk>\w+)/ticket_sublist$', LinesUpdateModalTicket.as_view(), name='CDNX_invoicing_saleslines_sublist_ticket'),

    url(r'^lines/(?P<pk>\w+)/ticketrectification_sublist$', LinesSubListTicketRectification.as_view(), name='CDNX_invoicing_saleslines_sublist_ticketrectificaction'),
    url(r'^lines/(?P<pk>\w+)/ticketrectification_sublist$', LinesUpdateModalTicketRectification.as_view(), name='CDNX_invoicing_saleslines_sublist_editmodal_ticketrectification'),

    # ###############
    url(r'^orders$', OrderList.as_view(), name='CDNX_invoicing_ordersaless_list'),
    url(r'^orders/add$', OrderCreate.as_view(), name='CDNX_invoicing_ordersaless_add'),
    url(r'^orders/addmodal$', OrderCreateModal.as_view(), name='CDNX_invoicing_ordersaless_addmodal'),

    url(r'^orders/addfrombudget$', OrderCreateModalFromBudget.as_view(), name='CDNX_invoicing_ordersaless_add_from_budget'),
    url(r'^orders/addfromshoppingcart$', OrderCreateModalFromShoppingCart.as_view(), name='CDNX_invoicing_ordersaless_add_from_shoppingcart'),

    url(r'^orders/(?P<pk>\w+)$', OrderDetails.as_view(), name='CDNX_invoicing_ordersaless_details'),
    url(r'^orders/(?P<pk>\w+)/edit$', OrderUpdate.as_view(), name='CDNX_invoicing_ordersaless_edit'),
    url(r'^orders/(?P<pk>\w+)/editmodal$', OrderUpdateModal.as_view(), name='CDNX_invoicing_ordersaless_editmodal'),
    url(r'^orders/(?P<pk>\w+)/delete$', OrderDelete.as_view(), name='CDNX_invoicing_ordersaless_delete'),
    url(r'^orders/(?P<pk>\w+)/status/(?P<action>\w+)$', OrderStatus.as_view(), name='CDNX_invoicing_ordersaless_status'),
    url(r'^orders/(?P<pk>\w+)/createalbaran$', OrderCreateAlbaran.as_view(), name='CDNX_invoicing_ordersaless_create_albaran'),
    url(r'^orders/(?P<pk>\w+)/createticket$', OrderCreateTicket.as_view(), name='CDNX_invoicing_ordersaless_create_ticket'),
    url(r'^orders/(?P<pk>\w+)/createinvoice$', OrderCreateInvoice.as_view(), name='CDNX_invoicing_ordersaless_create_invoice'),
    url(r'^orders/foreign/(?P<search>[\w\W]+|\*)$', OrderForeign.as_view(), name='CDNX_invoicing_ordersaless_foreign'),
    url(r'^orders/(?P<pk>\w+)/print$', OrderPrint.as_view(), name='CDNX_invoicing_ordersaless_print'),

    url(r'^salesorderdocuments/(?P<pk>\w+)/sublist$', OrderDocumentSubList.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist'),
    url(r'^salesorderdocuments/(?P<pk>\w+)/sublist/addmodal$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_addmodal'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_editmodal'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', OrderDocumentDelete.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_delete'),

    url(r'^printcounterdocumentbaskets/(?P<pk>\w+)/sublist$', PrintCounterDocumentBasketSublist.as_view(), name='CDNX_invoicing_printcounterdocumentbaskets_sublist'),
    url(r'^printcounterdocumentorders/(?P<pk>\w+)/sublist$', PrintCounterDocumentOrderSublist.as_view(), name='CDNX_invoicing_printcounterdocumentorders_sublist'),
    url(r'^printcounterdocumentalbarans/(?P<pk>\w+)/sublist$', PrintCounterDocumentAlbaranSublist.as_view(), name='CDNX_invoicing_printcounterdocumentalbarans_sublist'),
    url(r'^printcounterdocumenttickets/(?P<pk>\w+)/sublist$', PrintCounterDocumentTicketSublist.as_view(), name='CDNX_invoicing_printcounterdocumenttickets_sublist'),
    url(r'^printcounterdocumentticketrectifications/(?P<pk>\w+)/sublist$', PrintCounterDocumentTicketRectificationSublist.as_view(), name='CDNX_invoicing_printcounterdocumentticketrectifications_sublist'),
    url(r'^printcounterdocumentinvoices/(?P<pk>\w+)/sublist$', PrintCounterDocumentInvoiceSublist.as_view(), name='CDNX_invoicing_printcounterdocumentinvoices_sublist'),
    url(r'^printcounterdocumentinvoicerectifications/(?P<pk>\w+)/sublist$', PrintCounterDocumentInvoiceRectificationSublist.as_view(), name='CDNX_invoicing_printcounterdocumentinvoicerectifications_sublist'),

    url(r'^albarans$', AlbaranList.as_view(), name='CDNX_invoicing_albaransaless_list'),
    url(r'^albarans/add$', AlbaranCreate.as_view(), name='CDNX_invoicing_albaransaless_add'),
    url(r'^albarans/addmodal$', AlbaranCreateModal.as_view(), name='CDNX_invoicing_albaransaless_addmodal'),
    url(r'^albarans/(?P<pk>\w+)$', AlbaranDetails.as_view(), name='CDNX_invoicing_albaransaless_details'),
    url(r'^albarans/(?P<pk>\w+)/edit$', AlbaranUpdate.as_view(), name='CDNX_invoicing_albaransaless_edit'),
    url(r'^albarans/(?P<pk>\w+)/editmodal$', AlbaranUpdateModal.as_view(), name='CDNX_invoicing_albaransaless_editmodal'),
    url(r'^albarans/(?P<pk>\w+)/delete$', AlbaranDelete.as_view(), name='CDNX_invoicing_albaransaless_delete'),
    url(r'^albarans/(?P<pk>\w+)/createticket$', AlbaranCreateTicket.as_view(), name='CDNX_invoicing_albaransaless_create_ticket'),
    url(r'^albarans/(?P<pk>\w+)/createinvoice$', AlbaranCreateInvoice.as_view(), name='CDNX_invoicing_albaransaless_create_invoice'),
    url(r'^albarans/(?P<pk>\w+)/print$', AlbaranPrint.as_view(), name='CDNX_invoicing_albaransaless_print'),
    url(r'^albarans/(?P<pk>\w+)/send$', AlbaranSend.as_view(), name='CDNX_invoicing_albaransaless_send'),

    url(r'^invoices$', InvoiceList.as_view(), name='CDNX_invoicing_invoicesaless_list'),
    url(r'^invoices/add$', InvoiceCreate.as_view(), name='CDNX_invoicing_invoicesaless_add'),
    url(r'^invoices/addmodal$', InvoiceCreateModal.as_view(), name='CDNX_invoicing_invoicesaless_addmodal'),
    url(r'^invoices/(?P<pk>\w+)$', InvoiceDetails.as_view(), name='CDNX_invoicing_invoicesales_details'),
    url(r'^invoices/(?P<pk>\w+)/edit$', InvoiceUpdate.as_view(), name='CDNX_invoicing_invoicesaless_edit'),
    url(r'^invoices/(?P<pk>\w+)/editmodal$', InvoiceUpdateModal.as_view(), name='CDNX_invoicing_invoicesaless_editmodal'),
    url(r'^invoices/(?P<pk>\w+)/delete$', InvoiceDelete.as_view(), name='CDNX_invoicing_invoicesaless_delete'),
    url(r'^invoices/(?P<pk>\w+)/createinvoicerectification$', InvoiceCreateRectification.as_view(), name='CDNX_invoicing_invoicesaless_create_rectification'),
    url(r'^invoices/(?P<pk>\w+)/print$', InvoicePrint.as_view(), name='CDNX_invoicing_invoicesaless_print'),
    url(r'^invoices/(?P<ipk>\w+)/createinvoicerectification/(?P<pk>\w+)$', InvoiceCreateRectificationUnity.as_view(), name='CDNX_invoicing_invoicesaless_create_rectification_unity'),

    url(r'^invoicerectifications$', InvoiceRectificationList.as_view(), name='CDNX_invoicing_invoicerectificationsaless_list'),
    url(r'^invoicerectifications/add$', InvoiceRectificationCreate.as_view(), name='CDNX_invoicing_invoicerectificationsaless_add'),
    url(r'^invoicerectifications/addmodal$', InvoiceRectificationCreateModal.as_view(), name='CDNX_invoicing_invoicerectificationsaless_addmodal'),
    url(r'^invoicerectifications/(?P<pk>\w+)$', InvoiceRectificationDetails.as_view(), name='CDNX_invoicing_invoicerectificationsaless_detail'),
    url(r'^invoicerectifications/(?P<pk>\w+)/edit$', InvoiceRectificationUpdate.as_view(), name='CDNX_invoicing_invoicerectificationsaless_edit'),
    url(r'^invoicerectifications/(?P<pk>\w+)/editmodal$', InvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_invoicerectificationsaless_editmodal'),
    url(r'^invoicerectifications/(?P<pk>\w+)/delete$', InvoiceRectificationDelete.as_view(), name='CDNX_invoicing_invoicerectificationsaless_delete'),
    url(r'^invoicerectifications/(?P<pk>\w+)/print$', InvoiceRectificationPrint.as_view(), name='CDNX_invoicing_invoicerectificationsaless_print'),

    url(r'^tickets$', TicketList.as_view(), name='CDNX_invoicing_ticketsaless_list'),
    url(r'^tickets/add$', TicketCreate.as_view(), name='CDNX_invoicing_ticketsaless_add'),
    url(r'^tickets/addmodal$', TicketCreateModal.as_view(), name='CDNX_invoicing_ticketsaless_addmodal'),
    url(r'^tickets/(?P<pk>\w+)$', TicketDetails.as_view(), name='CDNX_invoicing_ticketsales_details'),
    url(r'^tickets/(?P<pk>\w+)/edit$', TicketUpdate.as_view(), name='CDNX_invoicing_ticketsaless_edit'),
    url(r'^tickets/(?P<pk>\w+)/editmodal$', TicketUpdateModal.as_view(), name='CDNX_invoicing_ticketsaless_editmodal'),
    url(r'^tickets/(?P<pk>\w+)/delete$', TicketDelete.as_view(), name='CDNX_invoicing_ticketsaless_delete'),
    url(r'^tickets/(?P<pk>\w+)/createinvoice$', TicketCreateInvoice.as_view(), name='CDNX_invoicing_ticketsaless_create_invoice'),
    url(r'^tickets/(?P<pk>\w+)/createticketrectification$', TicketCreateRectification.as_view(), name='CDNX_invoicing_ticketsaless_create_rectification'),
    url(r'^tickets/(?P<pk>\w+)/print$', TicketPrint.as_view(), name='CDNX_invoicing_ticketsaless_print'),

    url(r'^ticketrectifications$', TicketRectificationList.as_view(), name='CDNX_invoicing_ticketrectificationsaless_list'),
    url(r'^ticketrectifications/add$', TicketRectificationCreate.as_view(), name='CDNX_invoicing_ticketrectificationsaless_add'),
    url(r'^ticketrectifications/addmodal$', TicketRectificationCreateModal.as_view(), name='CDNX_invoicing_ticketrectificationsaless_addmodal'),
    url(r'^ticketrectifications/(?P<pk>\w+)$', TicketRectificationDetails.as_view(), name='CDNX_invoicing_invoicesales_details'),
    url(r'^ticketrectifications/(?P<pk>\w+)/edit$', TicketRectificationUpdate.as_view(), name='CDNX_invoicing_ticketrectificationsaless_edit'),
    url(r'^ticketrectifications/(?P<pk>\w+)/editmodal$', TicketRectificationUpdateModal.as_view(), name='CDNX_invoicing_ticketrectificationsaless_editmodal'),
    url(r'^ticketrectifications/(?P<pk>\w+)/delete$', TicketRectificationDelete.as_view(), name='CDNX_invoicing_ticketrectificationsaless_delete'),
    url(r'^ticketrectifications/(?P<pk>\w+)/print$', TicketRectificationPrint.as_view(), name='CDNX_invoicing_ticketrectificationsaless_print'),

    url(r'^reasonmodifications$', ReasonModificationList.as_view(), name='CDNX_invoicing_reasonmodifications_list'),
    url(r'^reasonmodifications/add$', ReasonModificationCreate.as_view(), name='CDNX_invoicing_reasonmodifications_add'),
    url(r'^reasonmodifications/addmodal$', ReasonModificationCreateModal.as_view(), name='CDNX_invoicing_reasonmodifications_addmodal'),
    url(r'^reasonmodifications/(?P<pk>\w+)$', ReasonModificationDetails.as_view(), name='CDNX_invoicing_reasonmodifications_details'),
    url(r'^reasonmodifications/(?P<pk>\w+)/edit$', ReasonModificationUpdate.as_view(), name='CDNX_invoicing_reasonmodifications_edit'),
    url(r'^reasonmodifications/(?P<pk>\w+)/editmodal$', ReasonModificationUpdateModal.as_view(), name='CDNX_invoicing_reasonmodifications_editmodal'),
    url(r'^reasonmodifications/(?P<pk>\w+)/delete$', ReasonModificationDelete.as_view(), name='CDNX_invoicing_reasonmodifications_delete'),
    url(r'^reasonmodifications/(?P<pk>\w+)/sublist$', ReasonModificationSubList.as_view(), name='CDNX_invoicing_reasonmodifications_sublist'),
    url(r'^reasonmodifications/(?P<pk>\w+)/sublist/add$', ReasonModificationCreateModal.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_add'),
    url(r'^reasonmodifications/(?P<pk>\w+)/sublist/addmodal$', ReasonModificationCreateModal.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_addmodal'),
    url(r'^reasonmodifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ReasonModificationDetailModal.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_details'),
    url(r'^reasonmodifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ReasonModificationUpdateModal.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_edit'),
    url(r'^reasonmodifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', ReasonModificationUpdateModal.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_editmodal'),
    url(r'^reasonmodifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ReasonModificationDelete.as_view(), name='CDNX_invoicing_reasonmodifications_sublist_delete'),

    # ####################

    url(r'^reasonmodificationlinebaskets/(?P<pk>\w+)/sublist$', ReasonModificationLineBasketSubList.as_view(), name='CDNX_invoicing_reasonmodificationlinebaskets_sublist'),
    url(r'^reasonmodificationlinebaskets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineBasketDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlinebaskets_sublist_modal'),

    url(r'^reasonmodificationlineorders/(?P<pk>\w+)/sublist$', ReasonModificationLineOrderSubList.as_view(), name='CDNX_invoicing_reasonmodificationlineorders_sublist'),
    url(r'^reasonmodificationlineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineOrderDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlineorders_sublist_modal'),

    url(r'^reasonmodificationlinealbarans/(?P<pk>\w+)/sublist$', ReasonModificationLineAlbaranSubList.as_view(), name='CDNX_invoicing_reasonmodificationlinealbarans_sublist'),
    url(r'^reasonmodificationlinealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineAlbaranDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlinealbarans_sublist_modal'),

    url(r'^reasonmodificationlinetickets/(?P<pk>\w+)/sublist$', ReasonModificationLineTicketSubList.as_view(), name='CDNX_invoicing_reasonmodificationlinetickets_sublist'),
    url(r'^reasonmodificationlinetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineTicketDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlinetickets_sublist_modal'),

    url(r'^reasonmodificationlineticketrectifications/(?P<pk>\w+)/sublist$', ReasonModificationLineTicketRectificationSubList.as_view(), name='CDNX_invoicing_reasonmodificationlineticketrectifications_sublist'),
    url(r'^reasonmodificationlineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineTicketRectificationDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlineticketrectifications_sublist_modal'),

    url(r'^reasonmodificationlineinvoices/(?P<pk>\w+)/sublist$', ReasonModificationLineInvoiceSubList.as_view(), name='CDNX_invoicing_reasonmodificationlineinvoices_sublist'),
    url(r'^reasonmodificationlineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ReasonModificationLineInvoiceDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlineinvoices_sublist_modal'),

    url(r'^reasonmodificationlineinvoicerectifications/(?P<pk>\w+)/sublist$', ReasonModificationLineInvoiceRectificationSubList.as_view(), name='CDNX_invoicing_reasonmodificationlineinvoicerectifications_sublist'),
    url(r'^reasonmodificationlineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$', ReasonModificationLineInvoiceRectificationDetailModal.as_view(), name='CDNX_invoicing_reasonmodificationlineinvoicerectifications_sublist_modal'),

]
"""
    url(r'^reservedproducts$', ReservedProductList.as_view(), name='CDNX_invoicing_reservedproducts_list'),
    url(r'^reservedproducts/add$', ReservedProductCreate.as_view(), name='CDNX_invoicing_reservedproducts_add'),
    url(r'^reservedproducts/(?P<pk>\w+)/edit$', ReservedProductUpdate.as_view(), name='CDNX_invoicing_reservedproducts_edit'),
    url(r'^reservedproducts/(?P<pk>\w+)/delete$', ReservedProductDelete.as_view(), name='CDNX_invoicing_reservedproducts_delete'),

    # url(r'^shoppingcarts/management$', ShoppingCartManagement.as_view(), name='CDNX_invoicing_shoppingcarts_management'),



"""


"""
    url(r'^linebaskets$', LineBasketList.as_view(), name='CDNX_invoicing_saleslinebaskets_list'),
    url(r'^linebaskets/add$', LineBasketCreate.as_view(), name='CDNX_invoicing_saleslinebaskets_add'),
    url(r'^linebaskets/addmodal$', LineBasketCreateModal.as_view(), name='CDNX_invoicing_saleslinebaskets_addmodal'),
    url(r'^linebaskets/(?P<pk>\w+)$', LineBasketDetails.as_view(), name='CDNX_invoicing_saleslinebaskets_details'),
    url(r'^linebaskets/(?P<pk>\w+)/edit$', LineBasketUpdate.as_view(), name='CDNX_invoicing_saleslinebaskets_edit'),
    url(r'^linebaskets/(?P<pk>\w+)/editmodal$', LineBasketUpdateModal.as_view(), name='CDNX_invoicing_saleslinebaskets_editmodal'),
    url(r'^linebaskets/(?P<pk>\w+)/sublist$', LineBasketSubList.as_view(), name='CDNX_invoicing_saleslinebaskets_sublist'),
    url(r'^linebaskets/(?P<pk>\w+)/sublist/addmodal$', LineBasketCreateModal.as_view(), name='CDNX_invoicing_saleslinebaskets_sublist_add'),
    # url(r'^linebaskets/(?P<pk>\w+)/sublist/addpackmodal$', LineBasketCreateModalPack.as_view(), name='CDNX_invoicing_saleslinebaskets_sublist_addpack'),
    url(r'^linebaskets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineBasketDetailModal.as_view(), name='CDNX_invoicing_saleslinebaskets_sublist_details'),
    url(r'^linebaskets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineBasketUpdateModal.as_view(), name='CDNX_invoicing_saleslinebaskets_sublist_details'),
    url(r'^linebaskets/foreign/(?P<search>[\w\W]+|\*)$', LineBasketForeign.as_view(), name='CDNX_invoicing_linebasketsaless_foreign'),

    url(r'^salesorderdocuments$', OrderDocumentList.as_view(), name='CDNX_invoicing_salesorderdocuments_list'),
    url(r'^salesorderdocuments/add$', OrderDocumentCreate.as_view(), name='CDNX_invoicing_salesorderdocuments_add'),
    url(r'^salesorderdocuments/addmodal$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_addmodal'),
    url(r'^salesorderdocuments/(?P<pk>\w+)$', OrderDocumentDetails.as_view(), name='CDNX_invoicing_salesorderdocuments_details'),
    url(r'^salesorderdocuments/(?P<pk>\w+)/edit$', OrderDocumentUpdate.as_view(), name='CDNX_invoicing_salesorderdocuments_edit'),
    url(r'^salesorderdocuments/(?P<pk>\w+)/editmodal$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_editmodal'),
    url(r'^salesorderdocuments/(?P<pk>\w+)/delete$', OrderDocumentDelete.as_view(), name='CDNX_invoicing_salesorderdocuments_delete'),

    url(r'^salesorderdocuments/(?P<pk>\w+)/sublist/add$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_add'),
    url(r'^salesorderdocuments/(?P<pk>\w+)/sublist/addmodal$', OrderDocumentCreateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_addmodal'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', OrderDocumentDetailModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_details'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_edit'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', OrderDocumentUpdateModal.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_editmodal'),
    url(r'^salesorderdocuments/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', OrderDocumentDelete.as_view(), name='CDNX_invoicing_salesorderdocuments_sublist_delete'),


    url(r'^lineorders$', LineOrderList.as_view(), name='CDNX_invoicing_lineordersaless_list'),
    url(r'^lineorders/(?P<pk>\w+)/delete$', LineOrderDelete.as_view(), name='CDNX_invoicing_lineordersaless_delete'),
    url(r'^lineorders/(?P<pk>\w+)/sublist$', LineOrderSubList.as_view(), name='CDNX_invoicing_lineordersaless_sublist'),
    url(r'^lineorders/(?P<pk>\w+)/sublist/addmodal$', LineOrderCreateModal.as_view(), name='CDNX_invoicing_lineordersaless_sublist_add'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineOrderDetailsModal.as_view(), name='CDNX_invoicing_lineordersaless_sublist_details'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineOrderUpdateModal.as_view(), name='CDNX_invoicing_lineordersaless_sublist_details'),
    url(r'^lineorders/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineOrderDelete.as_view(), name='CDNX_invoicing_lineordersaless_sublist_delete'),
    url(r'^lineorders/foreign/(?P<search>[\w\W]+|\*)$', LineOrderForeign.as_view(), name='CDNX_invoicing_lineordersaless_foreign'),
    url(r'^lineorders/foreigncustom/(?P<search>[\w\W]+|\*)$', LineOrderForeignCustom.as_view(), name='CDNX_invoicing_lineordersaless_foreign_custom'),


    url(r'^linealbarans$', LineAlbaranList.as_view(), name='CDNX_invoicing_linealbaransaless_list'),
    url(r'^linealbarans/(?P<pk>\w+)/delete$', LineAlbaranDelete.as_view(), name='CDNX_invoicing_linealbaransaless_delete'),
    url(r'^linealbarans/(?P<pk>\w+)/sublist$', LineAlbaranSubList.as_view(), name='CDNX_invoicing_linealbaransaless_sublist'),
    url(r'^linealbarans/(?P<pk>\w+)/sublist/addmodal$', LineAlbaranCreateModal.as_view(), name='CDNX_invoicing_linealbaransaless_sublist_add'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineAlbaranDetailsModal.as_view(), name='CDNX_invoicing_linealbaransaless_sublist_details'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineAlbaranUpdateModal.as_view(), name='CDNX_invoicing_linealbaransaless_sublist_details'),
    url(r'^linealbarans/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineAlbaranDelete.as_view(), name='CDNX_invoicing_linealbaransaless_sublist_delete'),
    url(r'^linealbarans/foreign/(?P<search>[\w\W]+|\*)$', LineAlbaranForeign.as_view(), name='CDNX_invoicing_linealbaransaless_foreign'),


    url(r'^linetickets$', LineTicketList.as_view(), name='CDNX_invoicing_lineticketsaless_list'),
    url(r'^linetickets/(?P<pk>\w+)/delete$', LineTicketDelete.as_view(), name='CDNX_invoicing_lineticketsaless_delete'),
    url(r'^linetickets/(?P<pk>\w+)/sublist$', LineTicketSubList.as_view(), name='CDNX_invoicing_lineticketsaless_sublist'),
    url(r'^linetickets/(?P<pk>\w+)/sublist/addmodal$', LineTicketCreateModal.as_view(), name='CDNX_invoicing_lineticketsaless_sublist_add'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineTicketDetailsModal.as_view(), name='CDNX_invoicing_lineticketsaless_sublist_details'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineTicketUpdateModal.as_view(), name='CDNX_invoicing_lineticketsaless_sublist_details'),
    url(r'^linetickets/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineTicketDelete.as_view(), name='CDNX_invoicing_lineticketsaless_sublist_delete'),
    url(r'^linetickets/foreign/(?P<search>[\w\W]+|\*)$', LineTicketForeign.as_view(), name='CDNX_invoicing_lineticketsaless_foreign'),


    url(r'^lineticketrectifications/(?P<pk>\w+)/sublist$', LineTicketRectificationSubList.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist'),
    url(r'^lineticketrectifications/(?P<pk>\w+)/sublist/addmodal$', LineTicketRectificationCreateModal.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist_add'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineTicketRectificationDetailModal.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist_details'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineTicketRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist_details'),
    url(r'^lineticketrectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineTicketRectificationDelete.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist_delete'),
    url(r'^lineticketrectifications/foreign/(?P<search>[\w\W]+|\*)$', LineTicketRectificationForeign.as_view(), name='CDNX_invoicing_lineticketrectificationsaless_sublist_foreign'),


    url(r'^lineinvoices$', LineInvoiceList.as_view(), name='CDNX_invoicing_lineinvoicesaless_list'),
    url(r'^lineinvoices/(?P<pk>\w+)/delete$', LineInvoiceDelete.as_view(), name='CDNX_invoicing_lineinvoicesaless_delete'),
    url(r'^lineinvoices/(?P<pk>\w+)/sublist$', LineInvoiceSubList.as_view(), name='CDNX_invoicing_lineinvoicesaless_sublist'),
    url(r'^lineinvoices/(?P<pk>\w+)/sublist/addmodal$', LineInvoiceCreateModal.as_view(), name='CDNX_invoicing_lineinvoicesaless_sublist_add'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineInvoiceDetailsModal.as_view(), name='CDNX_invoicing_lineinvoicesaless_sublist_details'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineInvoiceUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicesaless_sublist_details'),
    url(r'^lineinvoices/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineInvoiceDelete.as_view(), name='CDNX_invoicing_lineinvoicesaless_sublist_delete'),
    url(r'^lineinvoices/foreign/(?P<search>[\w\W]+|\*)$', LineInvoiceForeign.as_view(), name='CDNX_invoicing_lineinvoicessaless_foreign'),

    url(r'^lineinvoicerectifications/(?P<pk>\w+)/sublist$', LineInvoiceRectificationSubList.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist'),
    url(r'^lineinvoicerectifications/(?P<pk>\w+)/sublist/addmodal$', LineInvoiceRectificationCreateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist_add'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', LineInvoiceRectificationDetailModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist_details'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', LineInvoiceRectificationUpdateModal.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist_details'),
    url(r'^lineinvoicerectifications/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', LineInvoiceRectificationDelete.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist_delete'),
    url(r'^lineinvoicerectifications/foreign/(?P<search>[\w\W]+|\*)$', LineInvoiceRectificationForeign.as_view(), name='CDNX_invoicing_lineinvoicerectificationsaless_sublist_foreign'),



    # url(r'^lines$', LinesList.as_view(), name='CDNX_invoicing_saleslines_list'),
    # url(r'^lines/add$', LinesCreate.as_view(), name='CDNX_invoicing_saleslines_add'),
    # url(r'^lines/addmodal$', LinesCreateModal.as_view(), name='CDNX_invoicing_saleslines_addmodal'),
    # url(r'^lines/(?P<pk>\w+)$', LinesDetails.as_view(), name='CDNX_invoicing_saleslines_details'),
    # url(r'^lines/(?P<pk>\w+)/edit$', LinesUpdate.as_view(), name='CDNX_invoicing_saleslines_edit'),
    # url(r'^lines/(?P<pk>\w+)/editmodal$', LinesUpdateModal.as_view(), name='CDNX_invoicing_saleslines_editmodal'),
    # url(r'^lines/(?P<pk>\w+)/delete$', LinesDelete.as_view(), name='CDNX_invoicing_saleslines_delete'),
    # url(r'^lines/(?P<pk>\w+)/sublist/addpackmodal$', LinesCreateModalPack.as_view(), name='CDNX_invoicing_saleslines_sublist_addpack'),
    # url(r'^lines/(?P<cpk>\w+)/basket_sublist/(?P<pk>\w+)$', LinesDetailModalBasket.as_view(), name='CDNX_invoicing_saleslines_sublist_details'),
    # url(r'^lines/(?P<cpk>\w+)/basket_sublist/(?P<pk>\w+)/delete$', LinesDeleteBasket.as_view(), name='CDNX_invoicing_saleslines_sublist_delete'),
    # url(r'^lines/foreign/(?P<search>[\w\W]+|\*)$', LinesForeignBasket.as_view(), name='CDNX_invoicing_linesaless_foreign'),
"""



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-07-08 05:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0021_auto_20180523_1036'),
    ]

    operations = [
        migrations.AlterField(
            model_name='salesorder',
            name='status_order',
            field=models.CharField(choices=[('PE', 'Pending'), ('PA', 'Payment accepted'), ('SE', 'Sent'), ('DE', 'Delivered'), ('CA', 'Cancelled')], default='PE', max_length=2, verbose_name='Status'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-03-01 07:00
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('codenerix_pos', '0001_initial'),
        ('codenerix_invoicing', '0016_purchaseslinealbaran_product_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='CashDiary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('opened_date', models.DateTimeField(verbose_name='Opened Date')),
                ('opened_cash', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='Opened Cash')),
                ('opened_cash_extra', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='Opened Cash Deviation')),
                ('opened_cash_notes', models.TextField(blank=True, default='', verbose_name='Opened Cash Notes')),
                ('opened_cards', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='Opened Cards')),
                ('opened_cards_extra', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='Opened Cards Deviation')),
                ('opened_cards_notes', models.TextField(blank=True, default='', verbose_name='Opened Cards Notes')),
                ('closed_date', models.DateTimeField(blank=True, null=True, verbose_name='Closed Date')),
                ('closed_cash', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True, verbose_name='Closed Cash')),
                ('closed_cash_extra', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='Closed Cash Deviation')),
                ('closed_cash_notes', models.TextField(blank=True, default='', verbose_name='Closed Cash Notes')),
                ('closed_cards', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True, verbose_name='Closed Cards')),
                ('closed_cards_extra', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='Closed Cards Deviation')),
                ('closed_cards_notes', models.TextField(blank=True, default='', verbose_name='Closed Cards Notes')),
                ('closed_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='closed_cash_diarys', to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('opened_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='opened_cash_diarys', to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('pos', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cash_movements', to='codenerix_pos.POS', verbose_name='Point of Sales')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CashMovement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('kind', models.CharField(choices=[('TRA', 'Wire transfer'), ('CAR', 'Card'), ('CAS', 'Cash'), ('CRE', 'Credit'), ('PYP', 'Paypal'), ('30C', '30 day credit'), ('60C', '60 day credit'), ('90C', '90 day credit')], max_length=3, verbose_name='Kind')),
                ('kind_card', models.CharField(blank=True, choices=[('VIS', 'Visa'), ('MAS', 'MasterCard'), ('AME', 'American Express'), ('OTH', 'Other')], max_length=3, null=True, verbose_name='Kind Card')),
                ('date_movement', models.DateTimeField(verbose_name='Date of movement')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='Amount')),
                ('reason', models.TextField(blank=True, default=None, null=True, verbose_name='Reason')),
                ('cash_diary', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cash_movements', to='codenerix_invoicing.CashDiary', verbose_name='Cash diary')),
                ('order', models.ManyToManyField(related_name='cash_movements', to='codenerix_invoicing.SalesOrder', verbose_name='Sales orders')),
                ('pos_slot', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='cash_movements', to='codenerix_pos.POSSlot', verbose_name='Slot')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cash_movements', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-29 11:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0008_auto_20180126_1711'),
        ('codenerix_invoicing', '0004_saleslines'),
    ]

    operations = [
        migrations.CreateModel(
            name='Al',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('n', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='Fa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('n', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='P',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('a', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='test', to='codenerix_invoicing.Al', verbose_name='A')),
                ('f', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='test', to='codenerix_invoicing.Fa', verbose_name='F')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='Pr',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('n', models.IntegerField(blank=True, null=True, verbose_name='Parent pk')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.AddField(
            model_name='p',
            name='p',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='test', to='codenerix_invoicing.Pr', verbose_name='P'),
        ),
        migrations.AddField(
            model_name='p',
            name='product_final',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='test', to='codenerix_products.ProductFinal', verbose_name='Product'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-01 11:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0009_auto_20180201_1137'),
        ('codenerix_invoicing', '0011_reasonmodificationlinealbaran_reasonmodificationlinebasket_reasonmodificationlineinvoice_reasonmodif'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleslines',
            name='tax_basket_fk',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='lines_sales_basket', to='codenerix_products.TypeTax', verbose_name='Tax Basket'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='saleslines',
            name='tax_invoice_fk',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='lines_sales_invoice', to='codenerix_products.TypeTax', verbose_name='Tax Invoice'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='tax_order_fk',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='lines_sales_order', to='codenerix_products.TypeTax', verbose_name='Tax Sales order'),
        ),
        migrations.AddField(
            model_name='saleslines',
            name='tax_ticket_fk',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='lines_sales_ticket', to='codenerix_products.TypeTax', verbose_name='Tax Ticket'),
        ),
    ]
